@@ -8,13 +8,15 @@ describe("loop.store.ConversationStore", function () {
 
   var CALL_STATES = loop.store.CALL_STATES;
   var WS_STATES = loop.store.WS_STATES;
-  var FAILURE_REASONS = loop.shared.utils.FAILURE_REASONS;
+  var CALL_TYPES = loop.shared.utils.CALL_TYPES;
+  var WEBSOCKET_REASONS = loop.shared.utils.WEBSOCKET_REASONS;
+  var FAILURE_DETAILS = loop.shared.utils.FAILURE_DETAILS;
   var sharedActions = loop.shared.actions;
   var sharedUtils = loop.shared.utils;
   var sandbox, dispatcher, client, store, fakeSessionData, sdkDriver;
   var contact, fakeMozLoop;
   var connectPromise, resolveConnectPromise, rejectConnectPromise;
-  var wsCancelSpy, wsCloseSpy, wsMediaUpSpy, fakeWebsocket;
+  var wsCancelSpy, wsCloseSpy, wsDeclineSpy, wsMediaUpSpy, fakeWebsocket;
 
   function checkFailures(done, f) {
     try {
@@ -42,7 +44,8 @@ describe("loop.store.ConversationStore", function () {
       addConversationContext: sandbox.stub(),
       calls: {
         setCallInProgress: sandbox.stub(),
-        clearCallInProgress: sandbox.stub()
+        clearCallInProgress: sandbox.stub(),
+        blockDirectCaller: sandbox.stub()
       },
       rooms: {
         create: sandbox.stub()
@@ -62,11 +65,13 @@ describe("loop.store.ConversationStore", function () {
 
     wsCancelSpy = sinon.spy();
     wsCloseSpy = sinon.spy();
+    wsDeclineSpy = sinon.spy();
     wsMediaUpSpy = sinon.spy();
 
     fakeWebsocket = {
       cancel: wsCancelSpy,
       close: wsCloseSpy,
+      decline: wsDeclineSpy,
       mediaUp: wsMediaUpSpy
     };
 
@@ -140,7 +145,7 @@ describe("loop.store.ConversationStore", function () {
       store._isDesktop = true;
 
       store.connectionFailure(new sharedActions.ConnectionFailure({
-        reason: FAILURE_REASONS.UNABLE_TO_PUBLISH_MEDIA
+        reason: FAILURE_DETAILS.UNABLE_TO_PUBLISH_MEDIA
       }));
 
       sinon.assert.calledOnce(sdkDriver.retryPublishWithoutVideo);
@@ -150,7 +155,7 @@ describe("loop.store.ConversationStore", function () {
       store._isDesktop = true;
 
       store.connectionFailure(new sharedActions.ConnectionFailure({
-        reason: FAILURE_REASONS.UNABLE_TO_PUBLISH_MEDIA
+        reason: FAILURE_DETAILS.UNABLE_TO_PUBLISH_MEDIA
       }));
 
       expect(store.getStoreState().videoMuted).eql(true);
@@ -222,9 +227,12 @@ describe("loop.store.ConversationStore", function () {
       });
     });
 
-    describe("progress: connecting", function() {
+    describe("progress: connecting (outgoing calls)", function() {
       beforeEach(function() {
-        store.setStoreState({callState: CALL_STATES.ALERTING});
+        store.setStoreState({
+          callState: CALL_STATES.ALERTING,
+          outgoing: true
+        });
       });
 
       it("should change the state to 'ongoing'", function() {
@@ -244,7 +252,8 @@ describe("loop.store.ConversationStore", function () {
         sinon.assert.calledWithExactly(sdkDriver.connectSession, {
           apiKey: "fakeKey",
           sessionId: "321456",
-          sessionToken: "341256"
+          sessionToken: "341256",
+          sendTwoWayMediaTelemetry: true
         });
       });
 
@@ -259,6 +268,55 @@ describe("loop.store.ConversationStore", function () {
                                        "28", "321456", "142536");
       });
     });
+
+    describe("progress: connecting (incoming calls)", function() {
+      beforeEach(function() {
+        store.setStoreState({
+          callState: CALL_STATES.ALERTING,
+          outgoing: false,
+          windowId: 42
+        });
+
+        sandbox.stub(console, "error");
+        store._websocket = fakeWebsocket;
+      });
+
+      it("should log an error", function() {
+        store.connectionProgress(
+          new sharedActions.ConnectionProgress({wsState: WS_STATES.CONNECTING}));
+
+        sinon.assert.calledOnce(console.error);
+      });
+
+      it("should call decline on the websocket", function() {
+        store.connectionProgress(
+          new sharedActions.ConnectionProgress({wsState: WS_STATES.CONNECTING}));
+
+        sinon.assert.calledOnce(fakeWebsocket.decline);
+      });
+
+      it("should close the websocket", function() {
+        store.connectionProgress(
+          new sharedActions.ConnectionProgress({wsState: WS_STATES.CONNECTING}));
+
+        sinon.assert.calledOnce(fakeWebsocket.close);
+      });
+
+      it("should clear the call in progress for the backend", function() {
+        store.connectionProgress(
+          new sharedActions.ConnectionProgress({wsState: WS_STATES.CONNECTING}));
+
+        sinon.assert.calledOnce(fakeMozLoop.calls.clearCallInProgress);
+        sinon.assert.calledWithExactly(fakeMozLoop.calls.clearCallInProgress, 42);
+      });
+
+      it("should set the call state to CLOSE", function() {
+        store.connectionProgress(
+          new sharedActions.ConnectionProgress({wsState: WS_STATES.CONNECTING}));
+
+        expect(store.getStoreState("callState")).eql(CALL_STATES.CLOSE);
+      });
+    });
   });
 
   describe("#setupWindowData", function() {
@@ -298,6 +356,79 @@ describe("loop.store.ConversationStore", function () {
         .eql(sharedUtils.CALL_TYPES.AUDIO_VIDEO);
     });
 
+    describe("incoming calls", function() {
+      beforeEach(function() {
+        store.setStoreState({outgoing: false});
+      });
+
+      it("should initialize the websocket", function() {
+        sandbox.stub(loop, "CallConnectionWebSocket").returns({
+          promiseConnect: function() { return connectPromise; },
+          on: sinon.spy()
+        });
+
+        store.connectCall(
+          new sharedActions.ConnectCall({sessionData: fakeSessionData}));
+
+        sinon.assert.calledOnce(loop.CallConnectionWebSocket);
+        sinon.assert.calledWithExactly(loop.CallConnectionWebSocket, {
+          url: "fakeURL",
+          callId: "142536",
+          websocketToken: "543216"
+        });
+      });
+
+      it("should connect the websocket to the server", function() {
+        store.connectCall(
+          new sharedActions.ConnectCall({sessionData: fakeSessionData}));
+
+        sinon.assert.calledOnce(store._websocket.promiseConnect);
+      });
+
+      describe("WebSocket connection result", function() {
+        beforeEach(function() {
+          store.connectCall(
+            new sharedActions.ConnectCall({sessionData: fakeSessionData}));
+
+          sandbox.stub(dispatcher, "dispatch");
+        });
+
+        it("should dispatch a connection progress action on success", function(done) {
+          resolveConnectPromise(WS_STATES.INIT);
+
+          connectPromise.then(function() {
+            checkFailures(done, function() {
+              sinon.assert.calledOnce(dispatcher.dispatch);
+              // Can't use instanceof here, as that matches any action
+              sinon.assert.calledWithMatch(dispatcher.dispatch,
+                sinon.match.hasOwn("name", "connectionProgress"));
+              sinon.assert.calledWithMatch(dispatcher.dispatch,
+                sinon.match.hasOwn("wsState", WS_STATES.INIT));
+            });
+          }, function() {
+            done(new Error("Promise should have been resolve, not rejected"));
+          });
+        });
+
+        it("should dispatch a connection failure action on failure", function(done) {
+          rejectConnectPromise();
+
+          connectPromise.then(function() {
+            done(new Error("Promise should have been rejected, not resolved"));
+          }, function() {
+            checkFailures(done, function() {
+              sinon.assert.calledOnce(dispatcher.dispatch);
+              // Can't use instanceof here, as that matches any action
+              sinon.assert.calledWithMatch(dispatcher.dispatch,
+                sinon.match.hasOwn("name", "connectionFailure"));
+              sinon.assert.calledWithMatch(dispatcher.dispatch,
+                sinon.match.hasOwn("reason", "websocket-setup"));
+             });
+          });
+        });
+      });
+    });
+
     describe("outgoing calls", function() {
       it("should request the outgoing call data", function() {
         dispatcher.dispatch(
@@ -421,6 +552,101 @@ describe("loop.store.ConversationStore", function () {
     });
   });
 
+  describe("#acceptCall", function() {
+    beforeEach(function() {
+      store._websocket = {
+        accept: sinon.stub()
+      };
+    });
+
+    it("should save the call type", function() {
+      store.acceptCall(
+        new sharedActions.AcceptCall({callType: CALL_TYPES.AUDIO_ONLY}));
+
+      expect(store.getStoreState("callType")).eql(CALL_TYPES.AUDIO_ONLY);
+      expect(store.getStoreState("videoMuted")).eql(true);
+    });
+
+    it("should call accept on the websocket", function() {
+      store.acceptCall(
+        new sharedActions.AcceptCall({callType: CALL_TYPES.AUDIO_ONLY}));
+
+      sinon.assert.calledOnce(store._websocket.accept);
+    });
+
+    it("should change the state to 'ongoing'", function() {
+      store.acceptCall(
+        new sharedActions.AcceptCall({callType: CALL_TYPES.AUDIO_ONLY}));
+
+      expect(store.getStoreState("callState")).eql(CALL_STATES.ONGOING);
+    });
+
+    it("should connect the session with sendTwoWayMediaTelemetry set as falsy", function() {
+      store.setStoreState(fakeSessionData);
+
+      store.acceptCall(
+        new sharedActions.AcceptCall({callType: CALL_TYPES.AUDIO_ONLY}));
+
+      sinon.assert.calledOnce(sdkDriver.connectSession);
+      sinon.assert.calledWithExactly(sdkDriver.connectSession, {
+        apiKey: "fakeKey",
+        sessionId: "321456",
+        sessionToken: "341256",
+        sendTwoWayMediaTelemetry: undefined
+      });
+    });
+
+    it("should call mozLoop.addConversationContext", function() {
+      store.setStoreState(fakeSessionData);
+
+      store.acceptCall(
+        new sharedActions.AcceptCall({callType: CALL_TYPES.AUDIO_ONLY}));
+
+      sinon.assert.calledOnce(fakeMozLoop.addConversationContext);
+      sinon.assert.calledWithExactly(fakeMozLoop.addConversationContext,
+                                     "28", "321456", "142536");
+    });
+  });
+
+  describe("#declineCall", function() {
+    beforeEach(function() {
+      store._websocket = fakeWebsocket;
+
+      store.setStoreState({windowId: 42});
+    });
+
+    it("should block the caller if necessary", function() {
+      store.declineCall(new sharedActions.DeclineCall({blockCaller: true}));
+
+      sinon.assert.calledOnce(fakeMozLoop.calls.blockDirectCaller);
+    });
+
+    it("should call decline on the websocket", function() {
+      store.declineCall(new sharedActions.DeclineCall({blockCaller: false}));
+
+      sinon.assert.calledOnce(fakeWebsocket.decline);
+    });
+
+    it("should close the websocket", function() {
+      store.declineCall(new sharedActions.DeclineCall({blockCaller: false}));
+
+      sinon.assert.calledOnce(fakeWebsocket.close);
+    });
+
+    it("should clear the call in progress for the backend", function() {
+      store.declineCall(new sharedActions.DeclineCall({blockCaller: false}));
+
+      sinon.assert.calledOnce(fakeMozLoop.calls.clearCallInProgress);
+      sinon.assert.calledWithExactly(fakeMozLoop.calls.clearCallInProgress, 42);
+    });
+
+    it("should set the call state to CLOSE", function() {
+      store.declineCall(new sharedActions.DeclineCall({blockCaller: false}));
+
+      expect(store.getStoreState("callState")).eql(CALL_STATES.CLOSE);
+    });
+  });
+
   describe("#connectCall", function() {
     it("should save the call session data", function() {
       store.connectCall(
@@ -631,7 +857,9 @@ describe("loop.store.ConversationStore", function () {
       sinon.assert.calledOnce(sdkDriver.disconnectSession);
     });
 
-    it("should send a cancel message to the websocket if it is open", function() {
+    it("should send a cancel message to the websocket if it is open for outgoing calls", function() {
+      store.setStoreState({outgoing: true});
+
       store.cancelCall(new sharedActions.CancelCall());
 
       sinon.assert.calledOnce(wsCancelSpy);
@@ -769,6 +997,49 @@ describe("loop.store.ConversationStore", function () {
       });
   });
 
+  describe("#windowUnload", function() {
+    var fakeWebsocket;
+
+    beforeEach(function() {
+      fakeWebsocket = store._websocket = {
+        close: sinon.stub(),
+        decline: sinon.stub()
+      };
+
+      store.setStoreState({windowId: 42});
+    });
+
+    it("should decline the connection on the websocket for incoming calls if the state is alerting", function() {
+      store.setStoreState({
+        callState: CALL_STATES.ALERTING,
+        outgoing: false
+      });
+
+      store.windowUnload();
+
+      sinon.assert.calledOnce(fakeWebsocket.decline);
+    });
+
+    it("should disconnect the sdk session", function() {
+      store.windowUnload();
+
+      sinon.assert.calledOnce(sdkDriver.disconnectSession);
+    });
+
+    it("should close the websocket", function() {
+      store.windowUnload();
+
+      sinon.assert.calledOnce(fakeWebsocket.close);
+    });
+
+    it("should clear the call in progress for the backend", function() {
+      store.windowUnload();
+
+      sinon.assert.calledOnce(fakeMozLoop.calls.clearCallInProgress);
+      sinon.assert.calledWithExactly(fakeMozLoop.calls.clearCallInProgress, 42);
+    });
+  });
+
   describe("Events", function() {
     describe("Websocket progress", function() {
       beforeEach(function() {
@@ -778,10 +1049,14 @@ describe("loop.store.ConversationStore", function () {
         sandbox.stub(dispatcher, "dispatch");
       });
 
-      it("should dispatch a connection failure action on 'terminate'", function() {
+      it("should dispatch a connection failure action on 'terminate' for outgoing calls", function() {
+        store.setStoreState({
+          outgoing: true
+        });
+
         store._websocket.trigger("progress", {
           state: WS_STATES.TERMINATED,
-          reason: "reject"
+          reason: WEBSOCKET_REASONS.REJECT
         });
 
         sinon.assert.calledOnce(dispatcher.dispatch);
@@ -789,7 +1064,57 @@ describe("loop.store.ConversationStore", function () {
         sinon.assert.calledWithMatch(dispatcher.dispatch,
           sinon.match.hasOwn("name", "connectionFailure"));
         sinon.assert.calledWithMatch(dispatcher.dispatch,
-          sinon.match.hasOwn("reason", "reject"));
+          sinon.match.hasOwn("reason", WEBSOCKET_REASONS.REJECT));
+      });
+
+      it("should dispatch a connection failure action on 'terminate' for incoming calls if the previous state was not 'alerting' or 'init'", function() {
+        store.setStoreState({
+          outgoing: false
+        });
+
+        store._websocket.trigger("progress", {
+          state: WS_STATES.TERMINATED,
+          reason: WEBSOCKET_REASONS.CANCEL
+        }, WS_STATES.CONNECTING);
+
+        sinon.assert.calledOnce(dispatcher.dispatch);
+        // Can't use instanceof here, as that matches any action
+        sinon.assert.calledWithExactly(dispatcher.dispatch,
+          new sharedActions.ConnectionFailure({
+            reason: WEBSOCKET_REASONS.CANCEL
+          }));
+      });
+
+      it("should dispatch a cancel call action on 'terminate' for incoming calls if the previous state was 'init'", function() {
+        store.setStoreState({
+          outgoing: false
+        });
+
+        store._websocket.trigger("progress", {
+          state: WS_STATES.TERMINATED,
+          reason: WEBSOCKET_REASONS.CANCEL
+        }, WS_STATES.INIT);
+
+        sinon.assert.calledOnce(dispatcher.dispatch);
+        // Can't use instanceof here, as that matches any action
+        sinon.assert.calledWithExactly(dispatcher.dispatch,
+          new sharedActions.CancelCall({}));
+      });
+
+      it("should dispatch a cancel call action on 'terminate' for incoming calls if the previous state was 'alerting'", function() {
+        store.setStoreState({
+          outgoing: false
+        });
+
+        store._websocket.trigger("progress", {
+          state: WS_STATES.TERMINATED,
+          reason: WEBSOCKET_REASONS.CANCEL
+        }, WS_STATES.ALERTING);
+
+        sinon.assert.calledOnce(dispatcher.dispatch);
+        // Can't use instanceof here, as that matches any action
+        sinon.assert.calledWithExactly(dispatcher.dispatch,
+          new sharedActions.CancelCall({}));
       });
 
       it("should dispatch a connection progress action on 'alerting'", function() {
