@@ -1,6 +1,8 @@
 /* Any copyright is dedicated to the Public Domain.
    http://creativecommons.org/publicdomain/zero/1.0/ */
 
+"use strict";
+
 /**
  * Test that things behave reasonably when a reasonable Hawk-Session-Token
  * header is returned with the registration response.
@@ -16,7 +18,7 @@ add_test(function test_registration_returns_hawk_session_token() {
     response.finish();
   });
 
-  MozLoopService.register(mockPushHandler).then(() => {
+  MozLoopService.promiseRegisteredWithServers().then(() => {
     var hawkSessionPref;
     try {
       hawkSessionPref = Services.prefs.getCharPref("loop.hawk-session-token");
@@ -31,10 +33,11 @@ add_test(function test_registration_returns_hawk_session_token() {
   });
 });
 
-function run_test()
-{
+function run_test() {
   setupFakeLoopServer();
 
+  mockPushHandler.registrationPushURL = kEndPointUrl;
+
   do_register_cleanup(function() {
     Services.prefs.clearUserPref("loop.hawk-session-token");
   });
