@@ -16,6 +16,7 @@
 #include "nsStyleChangeList.h"
 #include "RestyleManager.h"
 #include "RestyleTrackerInlines.h"
+#include "nsTransitionManager.h"
 
 namespace mozilla {
 
@@ -39,7 +40,7 @@ FrameTagToString(dom::Element* aElement)
     nsFrame::ListTag(result, frame);
   } else {
     nsAutoString buf;
-    aElement->Tag()->ToString(buf);
+    aElement->NodeInfo()->NameAtom()->ToString(buf);
     result.AppendPrintf("(%s@%p)", NS_ConvertUTF16toUTF8(buf).get(), aElement);
   }
   return result;
@@ -82,6 +83,9 @@ CollectLaterSiblings(nsISupports* aElement,
 
 struct RestyleEnumerateData : RestyleTracker::Hints {
   nsRefPtr<dom::Element> mElement;
+#if defined(MOZ_ENABLE_PROFILER_SPS) && !defined(MOZILLA_XPCOMRT_API)
+  UniquePtr<ProfilerBacktrace> mBacktrace;
+#endif
 };
 
 struct RestyleCollector {
@@ -92,6 +96,31 @@ struct RestyleCollector {
 #endif
 };
 
+class RestyleTimelineMarker : public TimelineMarker
+{
+public:
+  RestyleTimelineMarker(nsDocShell* aDocShell,
+                        TracingMetadata aMetaData,
+                        nsRestyleHint aRestyleHint)
+    : TimelineMarker(aDocShell, "Styles", aMetaData)
+  {
+    if (aRestyleHint) {
+      mRestyleHint.AssignWithConversion(RestyleManager::RestyleHintToString(aRestyleHint));
+    }
+  }
+
+  virtual void AddDetails(JSContext* aCx,
+                          mozilla::dom::ProfileTimelineMarker& aMarker) override
+  {
+    if (GetMetaData() == TRACING_INTERVAL_START) {
+      aMarker.mRestyleHint.Construct(mRestyleHint);
+    }
+  }
+
+private:
+  nsAutoString mRestyleHint;
+};
+
 static PLDHashOperator
 CollectRestyles(nsISupports* aElement,
                 nsAutoPtr<RestyleTracker::RestyleData>& aData,
@@ -139,6 +168,9 @@ CollectRestyles(nsISupports* aElement,
   currentRestyle->mElement = element;
   currentRestyle->mRestyleHint = aData->mRestyleHint;
   currentRestyle->mChangeHint = aData->mChangeHint;
+#if defined(MOZ_ENABLE_PROFILER_SPS) && !defined(MOZILLA_XPCOMRT_API)
+  currentRestyle->mBacktrace = Move(aData->mBacktrace);
+#endif
 
 #ifdef RESTYLE_LOGGING
   collector->count++;
@@ -195,130 +227,206 @@ RestyleTracker::DoProcessRestyles()
   PROFILER_LABEL("RestyleTracker", "ProcessRestyles",
     js::ProfileEntry::Category::CSS);
 
-  mRestyleManager->BeginProcessingRestyles();
+  bool isTimelineRecording = false;
+  nsDocShell* docShell =
+    static_cast<nsDocShell*>(mRestyleManager->PresContext()->GetDocShell());
+  if (docShell) {
+    docShell->GetRecordProfileTimelineMarkers(&isTimelineRecording);
+  }
 
-  LOG_RESTYLE("Processing %d pending %srestyles with %d restyle roots for %s",
-              mPendingRestyles.Count(),
-              mRestyleManager->IsProcessingAnimationStyleChange()
-                ? (const char*) "animation " : (const char*) "",
-              static_cast<int>(mRestyleRoots.Length()),
-              GetDocumentURI(Document()).get());
-  LOG_RESTYLE_INDENT();
+  // Create a ReframingStyleContexts struct on the stack and put it in our
+  // mReframingStyleContexts for almost all of the remaining scope of
+  // this function.
+  //
+  // It needs to be *in* scope during BeginProcessingRestyles, which
+  // might (if mDoRebuildAllStyleData is true) do substantial amounts of
+  // restyle processing.
+  //
+  // However, it needs to be *out* of scope during
+  // EndProcessingRestyles, since we should release the style contexts
+  // it holds prior to any EndReconstruct call that
+  // EndProcessingRestyles makes.  This is because in EndReconstruct we
+  // try to destroy the old rule tree using the GC mechanism, which
+  // means it only gets destroyed if it's unreferenced (and if it's
+  // referenced, we assert).  So we want the ReframingStyleContexts
+  // (which holds old style contexts) to be destroyed before the
+  // EndReconstruct so those style contexts go away before
+  // EndReconstruct.
+  {
+    RestyleManager::ReframingStyleContexts
+      reframingStyleContexts(mRestyleManager);
 
-  // loop so that we process any restyle events generated by processing
-  while (mPendingRestyles.Count()) {
-    if (mHaveLaterSiblingRestyles) {
-      // Convert them to individual restyles on all the later siblings
-      nsAutoTArray<nsRefPtr<Element>, RESTYLE_ARRAY_STACKSIZE> laterSiblingArr;
-      LaterSiblingCollector siblingCollector = { this, &laterSiblingArr };
-      mPendingRestyles.Enumerate(CollectLaterSiblings, &siblingCollector);
-      for (uint32_t i = 0; i < laterSiblingArr.Length(); ++i) {
-        Element* element = laterSiblingArr[i];
-        for (nsIContent* sibling = element->GetNextSibling();
-             sibling;
-             sibling = sibling->GetNextSibling()) {
-          if (sibling->IsElement()) {
-            LOG_RESTYLE("adding pending restyle for %s due to "
-                        "eRestyle_LaterSiblings hint on %s",
-                        FrameTagToString(sibling->AsElement()).get(),
-                        FrameTagToString(element->AsElement()).get());
-            if (AddPendingRestyle(sibling->AsElement(), eRestyle_Subtree,
-                                  NS_STYLE_HINT_NONE)) {
-                // Nothing else to do here; we'll handle the following
-                // siblings when we get to |sibling| in laterSiblingArr.
-              break;
+    mRestyleManager->BeginProcessingRestyles(*this);
+
+    LOG_RESTYLE("Processing %d pending %srestyles with %d restyle roots for %s",
+                mPendingRestyles.Count(),
+                mRestyleManager->PresContext()->TransitionManager()->
+                  InAnimationOnlyStyleUpdate()
+                  ? (const char*) "animation " : (const char*) "",
+                static_cast<int>(mRestyleRoots.Length()),
+                GetDocumentURI(Document()).get());
+    LOG_RESTYLE_INDENT();
+
+    // loop so that we process any restyle events generated by processing
+    while (mPendingRestyles.Count()) {
+      if (mHaveLaterSiblingRestyles) {
+        // Convert them to individual restyles on all the later siblings
+        nsAutoTArray<nsRefPtr<Element>, RESTYLE_ARRAY_STACKSIZE> laterSiblingArr;
+        LaterSiblingCollector siblingCollector = { this, &laterSiblingArr };
+        mPendingRestyles.Enumerate(CollectLaterSiblings, &siblingCollector);
+        for (uint32_t i = 0; i < laterSiblingArr.Length(); ++i) {
+          Element* element = laterSiblingArr[i];
+          for (nsIContent* sibling = element->GetNextSibling();
+               sibling;
+               sibling = sibling->GetNextSibling()) {
+            if (sibling->IsElement()) {
+              LOG_RESTYLE("adding pending restyle for %s due to "
+                          "eRestyle_LaterSiblings hint on %s",
+                          FrameTagToString(sibling->AsElement()).get(),
+                          FrameTagToString(element->AsElement()).get());
+              if (AddPendingRestyle(sibling->AsElement(), eRestyle_Subtree,
+                                    NS_STYLE_HINT_NONE)) {
+                  // Nothing else to do here; we'll handle the following
+                  // siblings when we get to |sibling| in laterSiblingArr.
+                break;
+              }
             }
           }
         }
-      }
 
-      // Now remove all those eRestyle_LaterSiblings bits
-      for (uint32_t i = 0; i < laterSiblingArr.Length(); ++i) {
-        Element* element = laterSiblingArr[i];
-        NS_ASSERTION(element->HasFlag(RestyleBit()), "How did that happen?");
-        RestyleData* data;
+        // Now remove all those eRestyle_LaterSiblings bits
+        for (uint32_t i = 0; i < laterSiblingArr.Length(); ++i) {
+          Element* element = laterSiblingArr[i];
+          NS_ASSERTION(element->HasFlag(RestyleBit()), "How did that happen?");
+          RestyleData* data;
 #ifdef DEBUG
-        bool found =
+          bool found =
 #endif
-          mPendingRestyles.Get(element, &data);
-        NS_ASSERTION(found, "Where did our entry go?");
-        data->mRestyleHint =
-          nsRestyleHint(data->mRestyleHint & ~eRestyle_LaterSiblings);
+            mPendingRestyles.Get(element, &data);
+          NS_ASSERTION(found, "Where did our entry go?");
+          data->mRestyleHint =
+            nsRestyleHint(data->mRestyleHint & ~eRestyle_LaterSiblings);
+        }
+
+        LOG_RESTYLE("%d pending restyles after expanding out "
+                    "eRestyle_LaterSiblings", mPendingRestyles.Count());
+
+        mHaveLaterSiblingRestyles = false;
       }
 
-      LOG_RESTYLE("%d pending restyles after expanding out "
-                  "eRestyle_LaterSiblings", mPendingRestyles.Count());
+      uint32_t rootCount;
+      while ((rootCount = mRestyleRoots.Length())) {
+        // Make sure to pop the element off our restyle root array, so
+        // that we can freely append to the array as we process this
+        // element.
+        nsRefPtr<Element> element;
+        element.swap(mRestyleRoots[rootCount - 1]);
+        mRestyleRoots.RemoveElementAt(rootCount - 1);
 
-      mHaveLaterSiblingRestyles = false;
-    }
+        LOG_RESTYLE("processing style root %s at index %d",
+                    FrameTagToString(element).get(), rootCount - 1);
+        LOG_RESTYLE_INDENT();
 
-    uint32_t rootCount;
-    while ((rootCount = mRestyleRoots.Length())) {
-      // Make sure to pop the element off our restyle root array, so
-      // that we can freely append to the array as we process this
-      // element.
-      nsRefPtr<Element> element;
-      element.swap(mRestyleRoots[rootCount - 1]);
-      mRestyleRoots.RemoveElementAt(rootCount - 1);
+        // Do the document check before calling GetRestyleData, since we
+        // don't want to do the sibling-processing GetRestyleData does if
+        // the node is no longer relevant.
+        if (element->GetCrossShadowCurrentDoc() != Document()) {
+          // Content node has been removed from our document; nothing else
+          // to do here
+          LOG_RESTYLE("skipping, no longer in the document");
+          continue;
+        }
 
-      LOG_RESTYLE("processing style root %s at index %d",
-                  FrameTagToString(element).get(), rootCount - 1);
-      LOG_RESTYLE_INDENT();
+        nsAutoPtr<RestyleData> data;
+        if (!GetRestyleData(element, data)) {
+          LOG_RESTYLE("skipping, already restyled");
+          continue;
+        }
 
-      // Do the document check before calling GetRestyleData, since we
-      // don't want to do the sibling-processing GetRestyleData does if
-      // the node is no longer relevant.
-      if (element->GetCrossShadowCurrentDoc() != Document()) {
-        // Content node has been removed from our document; nothing else
-        // to do here
-        LOG_RESTYLE("skipping, no longer in the document");
+        if (isTimelineRecording) {
+          mozilla::UniquePtr<TimelineMarker> marker =
+            MakeUnique<RestyleTimelineMarker>(docShell,
+                                              TRACING_INTERVAL_START,
+                                              data->mRestyleHint);
+          docShell->AddProfileTimelineMarker(Move(marker));
+        }
+
+#if defined(MOZ_ENABLE_PROFILER_SPS) && !defined(MOZILLA_XPCOMRT_API)
+        Maybe<GeckoProfilerTracingRAII> profilerRAII;
+        if (profiler_feature_active("restyle")) {
+          profilerRAII.emplace("Paint", "Styles", Move(data->mBacktrace));
+        }
+#endif
+        ProcessOneRestyle(element, data->mRestyleHint, data->mChangeHint);
+        AddRestyleRootsIfAwaitingRestyle(data->mDescendants);
+
+        if (isTimelineRecording) {
+          mozilla::UniquePtr<TimelineMarker> marker =
+            MakeUnique<RestyleTimelineMarker>(docShell,
+                                              TRACING_INTERVAL_END,
+                                              data->mRestyleHint);
+          docShell->AddProfileTimelineMarker(Move(marker));
+        }
+      }
+
+      if (mHaveLaterSiblingRestyles) {
+        // Keep processing restyles for now
         continue;
       }
 
-      nsAutoPtr<RestyleData> data;
-      if (!GetRestyleData(element, data)) {
-        LOG_RESTYLE("skipping, already restyled");
-        continue;
-      }
+      // Now we only have entries with change hints left.  To be safe in
+      // case of reentry from the handing of the change hint, use a
+      // scratch array instead of calling out to ProcessOneRestyle while
+      // enumerating the hashtable.  Use the stack if we can, otherwise
+      // fall back on heap-allocation.
+      nsAutoTArray<RestyleEnumerateData, RESTYLE_ARRAY_STACKSIZE> restyleArr;
+      RestyleEnumerateData* restylesToProcess =
+        restyleArr.AppendElements(mPendingRestyles.Count());
+      if (restylesToProcess) {
+        RestyleEnumerateData* lastRestyle = restylesToProcess;
+        RestyleCollector collector = { this, &lastRestyle };
+        mPendingRestyles.Enumerate(CollectRestyles, &collector);
 
-      ProcessOneRestyle(element, data->mRestyleHint, data->mChangeHint);
-      AddRestyleRootsIfAwaitingRestyle(data->mDescendants);
-    }
-
-    if (mHaveLaterSiblingRestyles) {
-      // Keep processing restyles for now
-      continue;
-    }
-
-    // Now we only have entries with change hints left.  To be safe in
-    // case of reentry from the handing of the change hint, use a
-    // scratch array instead of calling out to ProcessOneRestyle while
-    // enumerating the hashtable.  Use the stack if we can, otherwise
-    // fall back on heap-allocation.
-    nsAutoTArray<RestyleEnumerateData, RESTYLE_ARRAY_STACKSIZE> restyleArr;
-    RestyleEnumerateData* restylesToProcess =
-      restyleArr.AppendElements(mPendingRestyles.Count());
-    if (restylesToProcess) {
-      RestyleEnumerateData* lastRestyle = restylesToProcess;
-      RestyleCollector collector = { this, &lastRestyle };
-      mPendingRestyles.Enumerate(CollectRestyles, &collector);
-
-      // Clear the hashtable now that we don't need it anymore
-      mPendingRestyles.Clear();
+        // Clear the hashtable now that we don't need it anymore
+        mPendingRestyles.Clear();
 
 #ifdef RESTYLE_LOGGING
-      uint32_t index = 0;
+        uint32_t index = 0;
 #endif
-      for (RestyleEnumerateData* currentRestyle = restylesToProcess;
-           currentRestyle != lastRestyle;
-           ++currentRestyle) {
-        LOG_RESTYLE("processing pending restyle %s at index %d/%d",
-                    FrameTagToString(currentRestyle->mElement).get(),
-                    index++, collector.count);
-        LOG_RESTYLE_INDENT();
-        ProcessOneRestyle(currentRestyle->mElement,
-                          currentRestyle->mRestyleHint,
-                          currentRestyle->mChangeHint);
+        for (RestyleEnumerateData* currentRestyle = restylesToProcess;
+             currentRestyle != lastRestyle;
+             ++currentRestyle) {
+          LOG_RESTYLE("processing pending restyle %s at index %d/%d",
+                      FrameTagToString(currentRestyle->mElement).get(),
+                      index++, collector.count);
+          LOG_RESTYLE_INDENT();
+
+#if defined(MOZ_ENABLE_PROFILER_SPS) && !defined(MOZILLA_XPCOMRT_API)
+          Maybe<GeckoProfilerTracingRAII> profilerRAII;
+          if (profiler_feature_active("restyle")) {
+            profilerRAII.emplace("Paint", "Styles", Move(currentRestyle->mBacktrace));
+          }
+#endif
+          if (isTimelineRecording) {
+            mozilla::UniquePtr<TimelineMarker> marker =
+              MakeUnique<RestyleTimelineMarker>(docShell,
+                                                TRACING_INTERVAL_START,
+                                                currentRestyle->mRestyleHint);
+            docShell->AddProfileTimelineMarker(Move(marker));
+          }
+
+          ProcessOneRestyle(currentRestyle->mElement,
+                            currentRestyle->mRestyleHint,
+                            currentRestyle->mChangeHint);
+
+          if (isTimelineRecording) {
+            mozilla::UniquePtr<TimelineMarker> marker =
+              MakeUnique<RestyleTimelineMarker>(docShell,
+                                                TRACING_INTERVAL_END,
+                                                currentRestyle->mRestyleHint);
+            docShell->AddProfileTimelineMarker(Move(marker));
+          }
+        }
       }
     }
   }
