@@ -9,29 +9,30 @@
 
 #include <algorithm>
 
-#include "nsCOMPtr.h"
-#include "nsFrameSelection.h"
-#include "nsIFrame.h"
-#include "nsIScrollableFrame.h"
-#include "nsIDOMNode.h"
-#include "nsISelection.h"
-#include "nsISelectionPrivate.h"
-#include "nsIContent.h"
-#include "nsIPresShell.h"
-#include "nsCanvasFrame.h"
-#include "nsPresContext.h"
 #include "nsBlockFrame.h"
-#include "nsISelectionController.h"
-#include "mozilla/Preferences.h"
-#include "mozilla/BasicEvents.h"
-#include "nsIDOMWindow.h"
-#include "nsQueryContentEventResult.h"
-#include "nsIInterfaceRequestorUtils.h"
-#include "nsView.h"
-#include "nsDOMTokenList.h"
+#include "nsCanvasFrame.h"
 #include "nsCaret.h"
-#include "mozilla/dom/CustomEvent.h"
+#include "nsCOMPtr.h"
 #include "nsContentUtils.h"
+#include "nsDOMTokenList.h"
+#include "nsFrameSelection.h"
+#include "nsIContent.h"
+#include "nsIDOMNode.h"
+#include "nsIDOMWindow.h"
+#include "nsIFrame.h"
+#include "nsIInterfaceRequestorUtils.h"
+#include "nsIPresShell.h"
+#include "nsIScrollableFrame.h"
+#include "nsISelection.h"
+#include "nsISelectionController.h"
+#include "nsISelectionPrivate.h"
+#include "nsPresContext.h"
+#include "nsQueryContentEventResult.h"
+#include "nsView.h"
+#include "mozilla/dom/SelectionStateChangedEvent.h"
+#include "mozilla/dom/CustomEvent.h"
+#include "mozilla/BasicEvents.h"
+#include "mozilla/Preferences.h"
 
 using namespace mozilla;
 
@@ -140,6 +141,16 @@ TouchCaret::GetCanvasFrame()
   return presShell->GetCanvasFrame();
 }
 
+nsIFrame*
+TouchCaret::GetRootFrame()
+{
+  nsCOMPtr<nsIPresShell> presShell = do_QueryReferent(mPresShell);
+  if (!presShell) {
+    return nullptr;
+  }
+  return presShell->GetRootFrame();
+}
+
 void
 TouchCaret::SetVisibility(bool aVisible)
 {
@@ -170,12 +181,6 @@ TouchCaret::SetVisibility(bool aVisible)
 
   // Set touch caret expiration time.
   mVisible ? LaunchExpirationTimer() : CancelExpirationTimer();
-
-  // We must call SetMayHaveTouchCaret() in order to get APZC to wait until the
-  // event has been round-tripped and check whether it has been handled,
-  // otherwise B2G will end up panning the document when the user tries to drag
-  // touch caret.
-  presShell->SetMayHaveTouchCaret(mVisible);
 }
 
 nsRect
@@ -244,7 +249,7 @@ TouchCaret::GetCaretYCenterPosition()
 }
 
 void
-TouchCaret::SetTouchFramePos(const nsPoint& aOrigin)
+TouchCaret::SetTouchFramePos(const nsRect& aCaretRect)
 {
   nsCOMPtr<nsIPresShell> presShell = do_QueryReferent(mPresShell);
   if (!presShell) {
@@ -258,14 +263,17 @@ TouchCaret::SetTouchFramePos(const nsPoint& aOrigin)
 
   // Convert aOrigin to CSS pixels.
   nsRefPtr<nsPresContext> presContext = presShell->GetPresContext();
-  int32_t x = presContext->AppUnitsToIntCSSPixels(aOrigin.x);
-  int32_t y = presContext->AppUnitsToIntCSSPixels(aOrigin.y);
+  int32_t x = presContext->AppUnitsToIntCSSPixels(aCaretRect.Center().x);
+  int32_t y = presContext->AppUnitsToIntCSSPixels(aCaretRect.y);
+  int32_t padding = presContext->AppUnitsToIntCSSPixels(aCaretRect.height);
 
   nsAutoString styleStr;
   styleStr.AppendLiteral("left: ");
   styleStr.AppendInt(x);
   styleStr.AppendLiteral("px; top: ");
   styleStr.AppendInt(y);
+  styleStr.AppendLiteral("px; padding-top: ");
+  styleStr.AppendInt(padding);
   styleStr.AppendLiteral("px;");
 
   TOUCHCARET_LOG("Set style: %s", NS_ConvertUTF16toUTF8(styleStr).get());
@@ -282,7 +290,6 @@ TouchCaret::MoveCaret(const nsPoint& movePoint)
   if (!focusFrame && !canvasFrame) {
     return;
   }
-
   nsIFrame* scrollable =
     nsLayoutUtils::GetClosestFrameOfType(focusFrame, nsGkAtoms::scrollFrame);
 
@@ -419,6 +426,12 @@ TouchCaret::IsDisplayable()
     return false;
   }
 
+  nsIFrame* rootFrame = GetRootFrame();
+  if (!rootFrame) {
+    TOUCHCARET_LOG("No root frame!");
+    return false;
+  }
+
   dom::Element* touchCaretElement = presShell->GetTouchCaretElement();
   if (!touchCaretElement) {
     TOUCHCARET_LOG("No touch caret frame element!");
@@ -458,7 +471,8 @@ TouchCaret::IsDisplayable()
     return false;
   }
 
-  if (!nsLayoutUtils::IsRectVisibleInScrollFrames(focusFrame, focusRect)) {
+  if (mState != TOUCHCARET_TOUCHDRAG_ACTIVE &&
+        !nsLayoutUtils::IsRectVisibleInScrollFrames(focusFrame, focusRect)) {
     TOUCHCARET_LOG("Caret does not show in the scrollable frame!");
     return false;
   }
@@ -472,34 +486,29 @@ TouchCaret::UpdatePosition()
 {
   MOZ_ASSERT(mVisible);
 
-  nsPoint pos = GetTouchCaretPosition();
-  pos = ClampPositionToScrollFrame(pos);
-  SetTouchFramePos(pos);
+  nsRect rect = GetTouchCaretRect();
+  rect = ClampRectToScrollFrame(rect);
+  SetTouchFramePos(rect);
 }
 
-nsPoint
-TouchCaret::GetTouchCaretPosition()
+nsRect
+TouchCaret::GetTouchCaretRect()
 {
   nsRect focusRect;
   nsIFrame* focusFrame = GetCaretFocusFrame(&focusRect);
-  nsIFrame* canvasFrame = GetCanvasFrame();
+  nsIFrame* rootFrame = GetRootFrame();
+  // Transform the position to make it relative to root frame.
+  nsLayoutUtils::TransformRect(focusFrame, rootFrame, focusRect);
 
-  // Position of the touch caret relative to focusFrame.
-  nsPoint pos = nsPoint(focusRect.x + (focusRect.width / 2),
-                        focusRect.y + focusRect.height);
-
-  // Transform the position to make it relative to canvas frame.
-  nsLayoutUtils::TransformPoint(focusFrame, canvasFrame, pos);
-
-  return pos;
+  return focusRect;
 }
 
-nsPoint
-TouchCaret::ClampPositionToScrollFrame(const nsPoint& aPosition)
+nsRect
+TouchCaret::ClampRectToScrollFrame(const nsRect& aRect)
 {
-  nsPoint pos = aPosition;
+  nsRect rect = aRect;
   nsIFrame* focusFrame = GetCaretFocusFrame();
-  nsIFrame* canvasFrame = GetCanvasFrame();
+  nsIFrame* rootFrame = GetRootFrame();
 
   // Clamp the touch caret position to the scrollframe boundary.
   nsIFrame* closestScrollFrame =
@@ -510,8 +519,8 @@ TouchCaret::ClampPositionToScrollFrame(const nsPoint& aPosition)
     nsRect visualRect = sf->GetScrollPortRect();
 
     // Clamp the touch caret in the scroll port.
-    nsLayoutUtils::TransformRect(closestScrollFrame, canvasFrame, visualRect);
-    pos = visualRect.ClampPoint(pos);
+    nsLayoutUtils::TransformRect(closestScrollFrame, rootFrame, visualRect);
+    rect = rect.Intersect(visualRect);
 
     // Get next ancestor scroll frame.
     closestScrollFrame =
@@ -519,7 +528,7 @@ TouchCaret::ClampPositionToScrollFrame(const nsPoint& aPosition)
                                            nsGkAtoms::scrollFrame);
   }
 
-  return pos;
+  return rect;
 }
 
 /* static */void
@@ -614,6 +623,12 @@ TouchCaret::HandleEvent(WidgetEvent* aEvent)
       TOUCHCARET_LOG("Receive key/wheel event %d", aEvent->message);
       SetVisibility(false);
       break;
+    case NS_MOUSE_MOZLONGTAP:
+      if (mState == TOUCHCARET_TOUCHDRAG_ACTIVE) {
+        // Disable long tap event from APZ while dragging the touch caret.
+        status = nsEventStatus_eConsumeNoDefault;
+      }
+      break;
     default:
       break;
   }
@@ -628,7 +643,7 @@ TouchCaret::GetEventPosition(WidgetTouchEvent* aEvent, int32_t aIdentifier)
     if (aEvent->touches[i]->mIdentifier == aIdentifier) {
       // Get event coordinate relative to canvas frame.
       nsIFrame* canvasFrame = GetCanvasFrame();
-      nsIntPoint touchIntPoint = aEvent->touches[i]->mRefPoint;
+      LayoutDeviceIntPoint touchIntPoint = aEvent->touches[i]->mRefPoint;
       return nsLayoutUtils::GetEventCoordinatesRelativeTo(aEvent,
                                                           touchIntPoint,
                                                           canvasFrame);
@@ -642,8 +657,7 @@ TouchCaret::GetEventPosition(WidgetMouseEvent* aEvent)
 {
   // Get event coordinate relative to canvas frame.
   nsIFrame* canvasFrame = GetCanvasFrame();
-  nsIntPoint mouseIntPoint =
-    LayoutDeviceIntPoint::ToUntyped(aEvent->AsGUIEvent()->refPoint);
+  LayoutDeviceIntPoint mouseIntPoint = aEvent->AsGUIEvent()->refPoint;
   return nsLayoutUtils::GetEventCoordinatesRelativeTo(aEvent,
                                                       mouseIntPoint,
                                                       canvasFrame);
@@ -924,25 +938,45 @@ TouchCaret::DispatchTapEvent()
     return;
   }
 
-  nsCOMPtr<nsIDocument> doc = presShell->GetDocument();
-  if (!doc) {
+  nsRefPtr<nsCaret> caret = presShell->GetCaret();
+  if (!caret) {
     return;
   }
 
-  ErrorResult res;
-  nsRefPtr<dom::Event> domEvent =
-    doc->CreateEvent(NS_LITERAL_STRING("CustomEvent"), res);
-  if (res.Failed()) {
+  dom::Selection* sel = static_cast<dom::Selection*>(caret->GetSelection());
+  if (!sel) {
     return;
   }
 
-  dom::CustomEvent* customEvent = static_cast<dom::CustomEvent*>(domEvent.get());
-  customEvent->InitCustomEvent(NS_LITERAL_STRING("touchcarettap"),
-                               true, false, nullptr);
-  customEvent->SetTrusted(true);
-  customEvent->GetInternalNSEvent()->mFlags.mOnlyChromeDispatch = true;
+  nsIDocument* doc = presShell->GetDocument();
+
+  MOZ_ASSERT(doc);
+
+  dom::SelectionStateChangedEventInit init;
+  init.mBubbles = true;
+
+  // XXX: Do we need to flush layout?
+  presShell->FlushPendingNotifications(Flush_Layout);
+  nsRect rect = nsContentUtils::GetSelectionBoundingRect(sel);
+  nsRefPtr<dom::DOMRect>domRect = new dom::DOMRect(ToSupports(doc));
+
+  domRect->SetLayoutRect(rect);
+  init.mBoundingClientRect = domRect;
+  init.mVisible = false;
+
+  sel->Stringify(init.mSelectedText);
+
+  dom::Sequence<dom::SelectionState> state;
+  state.AppendElement(dom::SelectionState::Taponcaret);
+  init.mStates = state;
+
+  nsRefPtr<dom::SelectionStateChangedEvent> event =
+    dom::SelectionStateChangedEvent::Constructor(doc, NS_LITERAL_STRING("mozselectionstatechanged"), init);
+
+  event->SetTrusted(true);
+  event->GetInternalNSEvent()->mFlags.mOnlyChromeDispatch = true;
   bool ret;
-  doc->DispatchEvent(domEvent, &ret);
+  doc->DispatchEvent(event, &ret);
 }
 
 void
