@@ -32,12 +32,12 @@
 #include "nsStyleSet.h"
 #include "nsContentUtils.h" // For AddScriptBlocker().
 #include "nsRefreshDriver.h"
+#include "TouchManager.h"
 #include "mozilla/Attributes.h"
 #include "mozilla/EventForwards.h"
 #include "mozilla/MemoryReporting.h"
 
 class nsRange;
-class nsIDragService;
 
 struct RangePaintInfo;
 struct nsCallbackEventRequest;
@@ -57,10 +57,11 @@ class EventDispatchingCallback;
 // to get the pref for any reason.
 #define PAINTLOCK_EVENT_DELAY 250
 
-class PresShell MOZ_FINAL : public nsIPresShell,
-                            public nsStubDocumentObserver,
-                            public nsISelectionController, public nsIObserver,
-                            public nsSupportsWeakReference
+class PresShell final : public nsIPresShell,
+                        public nsStubDocumentObserver,
+                        public nsISelectionController,
+                        public nsIObserver,
+                        public nsSupportsWeakReference
 {
 public:
   PresShell();
@@ -82,197 +83,196 @@ public:
   void Init(nsIDocument* aDocument, nsPresContext* aPresContext,
             nsViewManager* aViewManager, nsStyleSet* aStyleSet,
             nsCompatibility aCompatMode);
-  virtual void Destroy() MOZ_OVERRIDE;
-  virtual void MakeZombie() MOZ_OVERRIDE;
+  virtual void Destroy() override;
+  virtual void MakeZombie() override;
 
-  virtual nsresult SetPreferenceStyleRules(bool aForceReflow) MOZ_OVERRIDE;
+  virtual nsresult SetPreferenceStyleRules(bool aForceReflow) override;
 
-  NS_IMETHOD GetSelection(SelectionType aType, nsISelection** aSelection) MOZ_OVERRIDE;
-  virtual mozilla::dom::Selection* GetCurrentSelection(SelectionType aType) MOZ_OVERRIDE;
+  NS_IMETHOD GetSelection(SelectionType aType, nsISelection** aSelection) override;
+  virtual mozilla::dom::Selection* GetCurrentSelection(SelectionType aType) override;
 
-  NS_IMETHOD SetDisplaySelection(int16_t aToggle) MOZ_OVERRIDE;
-  NS_IMETHOD GetDisplaySelection(int16_t *aToggle) MOZ_OVERRIDE;
+  NS_IMETHOD SetDisplaySelection(int16_t aToggle) override;
+  NS_IMETHOD GetDisplaySelection(int16_t *aToggle) override;
   NS_IMETHOD ScrollSelectionIntoView(SelectionType aType, SelectionRegion aRegion,
-                                     int16_t aFlags) MOZ_OVERRIDE;
-  NS_IMETHOD RepaintSelection(SelectionType aType) MOZ_OVERRIDE;
+                                     int16_t aFlags) override;
+  NS_IMETHOD RepaintSelection(SelectionType aType) override;
 
-  virtual void BeginObservingDocument() MOZ_OVERRIDE;
-  virtual void EndObservingDocument() MOZ_OVERRIDE;
-  virtual nsresult Initialize(nscoord aWidth, nscoord aHeight) MOZ_OVERRIDE;
-  virtual nsresult ResizeReflow(nscoord aWidth, nscoord aHeight) MOZ_OVERRIDE;
-  virtual nsresult ResizeReflowOverride(nscoord aWidth, nscoord aHeight) MOZ_OVERRIDE;
-  virtual nsIPageSequenceFrame* GetPageSequenceFrame() const MOZ_OVERRIDE;
-  virtual nsCanvasFrame* GetCanvasFrame() const MOZ_OVERRIDE;
-  virtual nsIFrame* GetRealPrimaryFrameFor(nsIContent* aContent) const MOZ_OVERRIDE;
+  virtual void BeginObservingDocument() override;
+  virtual void EndObservingDocument() override;
+  virtual nsresult Initialize(nscoord aWidth, nscoord aHeight) override;
+  virtual nsresult ResizeReflow(nscoord aWidth, nscoord aHeight) override;
+  virtual nsresult ResizeReflowOverride(nscoord aWidth, nscoord aHeight) override;
+  virtual nsIPageSequenceFrame* GetPageSequenceFrame() const override;
+  virtual nsCanvasFrame* GetCanvasFrame() const override;
+  virtual nsIFrame* GetRealPrimaryFrameFor(nsIContent* aContent) const override;
 
-  virtual nsIFrame* GetPlaceholderFrameFor(nsIFrame* aFrame) const MOZ_OVERRIDE;
+  virtual nsIFrame* GetPlaceholderFrameFor(nsIFrame* aFrame) const override;
   virtual void FrameNeedsReflow(nsIFrame *aFrame, IntrinsicDirty aIntrinsicDirty,
-                                            nsFrameState aBitToAdd) MOZ_OVERRIDE;
-  virtual void FrameNeedsToContinueReflow(nsIFrame *aFrame) MOZ_OVERRIDE;
-  virtual void CancelAllPendingReflows() MOZ_OVERRIDE;
-  virtual bool IsSafeToFlush() const MOZ_OVERRIDE;
-  virtual void FlushPendingNotifications(mozFlushType aType) MOZ_OVERRIDE;
-  virtual void FlushPendingNotifications(mozilla::ChangesToFlush aType) MOZ_OVERRIDE;
+                                            nsFrameState aBitToAdd) override;
+  virtual void FrameNeedsToContinueReflow(nsIFrame *aFrame) override;
+  virtual void CancelAllPendingReflows() override;
+  virtual bool IsSafeToFlush() const override;
+  virtual void FlushPendingNotifications(mozFlushType aType) override;
+  virtual void FlushPendingNotifications(mozilla::ChangesToFlush aType) override;
   virtual void DestroyFramesFor(nsIContent*  aContent,
-                                nsIContent** aDestroyedFramesFor) MOZ_OVERRIDE;
-  virtual void CreateFramesFor(nsIContent* aContent) MOZ_OVERRIDE;
+                                nsIContent** aDestroyedFramesFor) override;
+  virtual void CreateFramesFor(nsIContent* aContent) override;
 
   /**
    * Recreates the frames for a node
    */
-  virtual nsresult RecreateFramesFor(nsIContent* aContent) MOZ_OVERRIDE;
+  virtual nsresult RecreateFramesFor(nsIContent* aContent) override;
 
   /**
    * Post a callback that should be handled after reflow has finished.
    */
-  virtual nsresult PostReflowCallback(nsIReflowCallback* aCallback) MOZ_OVERRIDE;
-  virtual void CancelReflowCallback(nsIReflowCallback* aCallback) MOZ_OVERRIDE;
+  virtual nsresult PostReflowCallback(nsIReflowCallback* aCallback) override;
+  virtual void CancelReflowCallback(nsIReflowCallback* aCallback) override;
 
-  virtual void ClearFrameRefs(nsIFrame* aFrame) MOZ_OVERRIDE;
-  virtual already_AddRefed<gfxContext> CreateReferenceRenderingContext() MOZ_OVERRIDE;
+  virtual void ClearFrameRefs(nsIFrame* aFrame) override;
+  virtual already_AddRefed<gfxContext> CreateReferenceRenderingContext() override;
   virtual nsresult GoToAnchor(const nsAString& aAnchorName, bool aScroll,
-                              uint32_t aAdditionalScrollFlags = 0) MOZ_OVERRIDE;
-  virtual nsresult ScrollToAnchor() MOZ_OVERRIDE;
+                              uint32_t aAdditionalScrollFlags = 0) override;
+  virtual nsresult ScrollToAnchor() override;
 
   virtual nsresult ScrollContentIntoView(nsIContent* aContent,
                                                      ScrollAxis  aVertical,
                                                      ScrollAxis  aHorizontal,
-                                                     uint32_t    aFlags) MOZ_OVERRIDE;
+                                                     uint32_t    aFlags) override;
   virtual bool ScrollFrameRectIntoView(nsIFrame*     aFrame,
                                        const nsRect& aRect,
                                        ScrollAxis    aVertical,
                                        ScrollAxis    aHorizontal,
-                                       uint32_t      aFlags) MOZ_OVERRIDE;
+                                       uint32_t      aFlags) override;
   virtual nsRectVisibility GetRectVisibility(nsIFrame *aFrame,
                                              const nsRect &aRect,
-                                             nscoord aMinTwips) const MOZ_OVERRIDE;
+                                             nscoord aMinTwips) const override;
 
-  virtual void SetIgnoreFrameDestruction(bool aIgnore) MOZ_OVERRIDE;
-  virtual void NotifyDestroyingFrame(nsIFrame* aFrame) MOZ_OVERRIDE;
+  virtual void SetIgnoreFrameDestruction(bool aIgnore) override;
+  virtual void NotifyDestroyingFrame(nsIFrame* aFrame) override;
 
-  virtual nsresult CaptureHistoryState(nsILayoutHistoryState** aLayoutHistoryState) MOZ_OVERRIDE;
+  virtual nsresult CaptureHistoryState(nsILayoutHistoryState** aLayoutHistoryState) override;
 
-  virtual void UnsuppressPainting() MOZ_OVERRIDE;
+  virtual void UnsuppressPainting() override;
 
-  virtual nsresult GetAgentStyleSheets(nsCOMArray<nsIStyleSheet>& aSheets) MOZ_OVERRIDE;
-  virtual nsresult SetAgentStyleSheets(const nsCOMArray<nsIStyleSheet>& aSheets) MOZ_OVERRIDE;
+  virtual nsresult GetAgentStyleSheets(nsCOMArray<nsIStyleSheet>& aSheets) override;
+  virtual nsresult SetAgentStyleSheets(const nsCOMArray<nsIStyleSheet>& aSheets) override;
 
-  virtual nsresult AddOverrideStyleSheet(nsIStyleSheet *aSheet) MOZ_OVERRIDE;
-  virtual nsresult RemoveOverrideStyleSheet(nsIStyleSheet *aSheet) MOZ_OVERRIDE;
+  virtual nsresult AddOverrideStyleSheet(nsIStyleSheet *aSheet) override;
+  virtual nsresult RemoveOverrideStyleSheet(nsIStyleSheet *aSheet) override;
 
   virtual nsresult HandleEventWithTarget(
                                  mozilla::WidgetEvent* aEvent,
                                  nsIFrame* aFrame,
                                  nsIContent* aContent,
-                                 nsEventStatus* aStatus) MOZ_OVERRIDE;
-  virtual nsIFrame* GetEventTargetFrame() MOZ_OVERRIDE;
+                                 nsEventStatus* aStatus) override;
+  virtual nsIFrame* GetEventTargetFrame() override;
   virtual already_AddRefed<nsIContent> GetEventTargetContent(
-                                                     mozilla::WidgetEvent* aEvent) MOZ_OVERRIDE;
+                                                     mozilla::WidgetEvent* aEvent) override;
 
-  virtual void NotifyCounterStylesAreDirty() MOZ_OVERRIDE;
+  virtual void NotifyCounterStylesAreDirty() override;
 
-  virtual nsresult ReconstructFrames(void) MOZ_OVERRIDE;
-  virtual void Freeze() MOZ_OVERRIDE;
-  virtual void Thaw() MOZ_OVERRIDE;
-  virtual void FireOrClearDelayedEvents(bool aFireEvents) MOZ_OVERRIDE;
+  virtual nsresult ReconstructFrames(void) override;
+  virtual void Freeze() override;
+  virtual void Thaw() override;
+  virtual void FireOrClearDelayedEvents(bool aFireEvents) override;
 
   virtual nsresult RenderDocument(const nsRect& aRect, uint32_t aFlags,
                                               nscolor aBackgroundColor,
-                                              gfxContext* aThebesContext) MOZ_OVERRIDE;
+                                              gfxContext* aThebesContext) override;
 
   virtual mozilla::TemporaryRef<SourceSurface>
   RenderNode(nsIDOMNode* aNode,
              nsIntRegion* aRegion,
              nsIntPoint& aPoint,
-             nsIntRect* aScreenRect) MOZ_OVERRIDE;
+             nsIntRect* aScreenRect) override;
 
   virtual mozilla::TemporaryRef<SourceSurface>
   RenderSelection(nsISelection* aSelection,
                   nsIntPoint& aPoint,
-                  nsIntRect* aScreenRect) MOZ_OVERRIDE;
+                  nsIntRect* aScreenRect) override;
 
-  virtual already_AddRefed<nsPIDOMWindow> GetRootWindow() MOZ_OVERRIDE;
+  virtual already_AddRefed<nsPIDOMWindow> GetRootWindow() override;
 
-  virtual LayerManager* GetLayerManager() MOZ_OVERRIDE;
+  virtual LayerManager* GetLayerManager() override;
 
-  virtual void SetIgnoreViewportScrolling(bool aIgnore) MOZ_OVERRIDE;
+  virtual void SetIgnoreViewportScrolling(bool aIgnore) override;
 
-  virtual nsresult SetResolution(float aXResolution, float aYResolution) MOZ_OVERRIDE {
-    return SetResolutionImpl(aXResolution, aYResolution, /* aScaleToResolution = */ false);
+  virtual nsresult SetResolution(float aResolution) override {
+    return SetResolutionImpl(aResolution, /* aScaleToResolution = */ false);
   }
-  virtual nsresult SetResolutionAndScaleTo(float aXResolution, float aYResolution) MOZ_OVERRIDE {
-    return SetResolutionImpl(aXResolution, aYResolution, /* aScaleToResolution = */ true);
+  virtual nsresult SetResolutionAndScaleTo(float aResolution) override {
+    return SetResolutionImpl(aResolution, /* aScaleToResolution = */ true);
   }
-  virtual bool ScaleToResolution() const MOZ_OVERRIDE;
-  virtual gfxSize GetCumulativeResolution() MOZ_OVERRIDE;
+  virtual bool ScaleToResolution() const override;
+  virtual float GetCumulativeResolution() override;
 
   //nsIViewObserver interface
 
   virtual void Paint(nsView* aViewToPaint, const nsRegion& aDirtyRegion,
-                     uint32_t aFlags) MOZ_OVERRIDE;
+                     uint32_t aFlags) override;
   virtual nsresult HandleEvent(nsIFrame* aFrame,
                                mozilla::WidgetGUIEvent* aEvent,
                                bool aDontRetargetEvents,
-                               nsEventStatus* aEventStatus) MOZ_OVERRIDE;
+                               nsEventStatus* aEventStatus,
+                               nsIContent** aTargetContent) override;
   virtual nsresult HandleDOMEventWithTarget(
                                  nsIContent* aTargetContent,
                                  mozilla::WidgetEvent* aEvent,
-                                 nsEventStatus* aStatus) MOZ_OVERRIDE;
+                                 nsEventStatus* aStatus) override;
   virtual nsresult HandleDOMEventWithTarget(nsIContent* aTargetContent,
                                                         nsIDOMEvent* aEvent,
-                                                        nsEventStatus* aStatus) MOZ_OVERRIDE;
-  virtual bool ShouldIgnoreInvalidation() MOZ_OVERRIDE;
-  virtual void WillPaint() MOZ_OVERRIDE;
-  virtual void WillPaintWindow() MOZ_OVERRIDE;
-  virtual void DidPaintWindow() MOZ_OVERRIDE;
-  virtual void ScheduleViewManagerFlush(PaintType aType = PAINT_DEFAULT) MOZ_OVERRIDE;
+                                                        nsEventStatus* aStatus) override;
+  virtual bool ShouldIgnoreInvalidation() override;
+  virtual void WillPaint() override;
+  virtual void WillPaintWindow() override;
+  virtual void DidPaintWindow() override;
+  virtual void ScheduleViewManagerFlush(PaintType aType = PAINT_DEFAULT) override;
   virtual void DispatchSynthMouseMove(mozilla::WidgetGUIEvent* aEvent,
-                                      bool aFlushOnHoverChange) MOZ_OVERRIDE;
-  virtual void ClearMouseCaptureOnView(nsView* aView) MOZ_OVERRIDE;
-  virtual bool IsVisible() MOZ_OVERRIDE;
+                                      bool aFlushOnHoverChange) override;
+  virtual void ClearMouseCaptureOnView(nsView* aView) override;
+  virtual bool IsVisible() override;
 
   // touch caret
-  virtual already_AddRefed<mozilla::TouchCaret> GetTouchCaret() const MOZ_OVERRIDE;
-  virtual mozilla::dom::Element* GetTouchCaretElement() const MOZ_OVERRIDE;
-  virtual void SetMayHaveTouchCaret(bool aSet) MOZ_OVERRIDE;
-  virtual bool MayHaveTouchCaret() MOZ_OVERRIDE;
+  virtual already_AddRefed<mozilla::TouchCaret> GetTouchCaret() const override;
+  virtual mozilla::dom::Element* GetTouchCaretElement() const override;
   // selection caret
-  virtual already_AddRefed<mozilla::SelectionCarets> GetSelectionCarets() const MOZ_OVERRIDE;
-  virtual mozilla::dom::Element* GetSelectionCaretsStartElement() const MOZ_OVERRIDE;
-  virtual mozilla::dom::Element* GetSelectionCaretsEndElement() const MOZ_OVERRIDE;
+  virtual already_AddRefed<mozilla::SelectionCarets> GetSelectionCarets() const override;
+  virtual mozilla::dom::Element* GetSelectionCaretsStartElement() const override;
+  virtual mozilla::dom::Element* GetSelectionCaretsEndElement() const override;
   // caret handling
-  virtual already_AddRefed<nsCaret> GetCaret() const MOZ_OVERRIDE;
-  NS_IMETHOD SetCaretEnabled(bool aInEnable) MOZ_OVERRIDE;
-  NS_IMETHOD SetCaretReadOnly(bool aReadOnly) MOZ_OVERRIDE;
-  NS_IMETHOD GetCaretEnabled(bool *aOutEnabled) MOZ_OVERRIDE;
-  NS_IMETHOD SetCaretVisibilityDuringSelection(bool aVisibility) MOZ_OVERRIDE;
-  NS_IMETHOD GetCaretVisible(bool *_retval) MOZ_OVERRIDE;
-  virtual void SetCaret(nsCaret *aNewCaret) MOZ_OVERRIDE;
-  virtual void RestoreCaret() MOZ_OVERRIDE;
+  virtual already_AddRefed<nsCaret> GetCaret() const override;
+  NS_IMETHOD SetCaretEnabled(bool aInEnable) override;
+  NS_IMETHOD SetCaretReadOnly(bool aReadOnly) override;
+  NS_IMETHOD GetCaretEnabled(bool *aOutEnabled) override;
+  NS_IMETHOD SetCaretVisibilityDuringSelection(bool aVisibility) override;
+  NS_IMETHOD GetCaretVisible(bool *_retval) override;
+  virtual void SetCaret(nsCaret *aNewCaret) override;
+  virtual void RestoreCaret() override;
 
-  NS_IMETHOD SetSelectionFlags(int16_t aInEnable) MOZ_OVERRIDE;
-  NS_IMETHOD GetSelectionFlags(int16_t *aOutEnable) MOZ_OVERRIDE;
+  NS_IMETHOD SetSelectionFlags(int16_t aInEnable) override;
+  NS_IMETHOD GetSelectionFlags(int16_t *aOutEnable) override;
 
   // nsISelectionController
 
-  NS_IMETHOD PhysicalMove(int16_t aDirection, int16_t aAmount, bool aExtend) MOZ_OVERRIDE;
-  NS_IMETHOD CharacterMove(bool aForward, bool aExtend) MOZ_OVERRIDE;
-  NS_IMETHOD CharacterExtendForDelete() MOZ_OVERRIDE;
-  NS_IMETHOD CharacterExtendForBackspace() MOZ_OVERRIDE;
-  NS_IMETHOD WordMove(bool aForward, bool aExtend) MOZ_OVERRIDE;
-  NS_IMETHOD WordExtendForDelete(bool aForward) MOZ_OVERRIDE;
-  NS_IMETHOD LineMove(bool aForward, bool aExtend) MOZ_OVERRIDE;
-  NS_IMETHOD IntraLineMove(bool aForward, bool aExtend) MOZ_OVERRIDE;
-  NS_IMETHOD PageMove(bool aForward, bool aExtend) MOZ_OVERRIDE;
-  NS_IMETHOD ScrollPage(bool aForward) MOZ_OVERRIDE;
-  NS_IMETHOD ScrollLine(bool aForward) MOZ_OVERRIDE;
-  NS_IMETHOD ScrollCharacter(bool aRight) MOZ_OVERRIDE;
-  NS_IMETHOD CompleteScroll(bool aForward) MOZ_OVERRIDE;
-  NS_IMETHOD CompleteMove(bool aForward, bool aExtend) MOZ_OVERRIDE;
-  NS_IMETHOD SelectAll() MOZ_OVERRIDE;
-  NS_IMETHOD CheckVisibility(nsIDOMNode *node, int16_t startOffset, int16_t EndOffset, bool *_retval) MOZ_OVERRIDE;
+  NS_IMETHOD PhysicalMove(int16_t aDirection, int16_t aAmount, bool aExtend) override;
+  NS_IMETHOD CharacterMove(bool aForward, bool aExtend) override;
+  NS_IMETHOD CharacterExtendForDelete() override;
+  NS_IMETHOD CharacterExtendForBackspace() override;
+  NS_IMETHOD WordMove(bool aForward, bool aExtend) override;
+  NS_IMETHOD WordExtendForDelete(bool aForward) override;
+  NS_IMETHOD LineMove(bool aForward, bool aExtend) override;
+  NS_IMETHOD IntraLineMove(bool aForward, bool aExtend) override;
+  NS_IMETHOD PageMove(bool aForward, bool aExtend) override;
+  NS_IMETHOD ScrollPage(bool aForward) override;
+  NS_IMETHOD ScrollLine(bool aForward) override;
+  NS_IMETHOD ScrollCharacter(bool aRight) override;
+  NS_IMETHOD CompleteScroll(bool aForward) override;
+  NS_IMETHOD CompleteMove(bool aForward, bool aExtend) override;
+  NS_IMETHOD SelectAll() override;
+  NS_IMETHOD CheckVisibility(nsIDOMNode *node, int16_t startOffset, int16_t EndOffset, bool *_retval) override;
   virtual nsresult CheckVisibilityContent(nsIContent* aNode, int16_t aStartOffset,
-                                          int16_t aEndOffset, bool* aRetval) MOZ_OVERRIDE;
+                                          int16_t aEndOffset, bool* aRetval) override;
 
   // nsIDocumentObserver
   NS_DECL_NSIDOCUMENTOBSERVER_BEGINUPDATE
@@ -299,69 +299,69 @@ public:
   NS_DECL_NSIOBSERVER
 
 #ifdef MOZ_REFLOW_PERF
-  virtual void DumpReflows() MOZ_OVERRIDE;
-  virtual void CountReflows(const char * aName, nsIFrame * aFrame) MOZ_OVERRIDE;
+  virtual void DumpReflows() override;
+  virtual void CountReflows(const char * aName, nsIFrame * aFrame) override;
   virtual void PaintCount(const char * aName,
                                       nsRenderingContext* aRenderingContext,
                                       nsPresContext* aPresContext,
                                       nsIFrame * aFrame,
                                       const nsPoint& aOffset,
-                                      uint32_t aColor) MOZ_OVERRIDE;
-  virtual void SetPaintFrameCount(bool aOn) MOZ_OVERRIDE;
-  virtual bool IsPaintingFrameCounts() MOZ_OVERRIDE;
+                                      uint32_t aColor) override;
+  virtual void SetPaintFrameCount(bool aOn) override;
+  virtual bool IsPaintingFrameCounts() override;
 #endif
 
 #ifdef DEBUG
   virtual void ListStyleContexts(nsIFrame *aRootFrame, FILE *out,
-                                 int32_t aIndent = 0) MOZ_OVERRIDE;
+                                 int32_t aIndent = 0) override;
 
-  virtual void ListStyleSheets(FILE *out, int32_t aIndent = 0) MOZ_OVERRIDE;
-  virtual void VerifyStyleTree() MOZ_OVERRIDE;
+  virtual void ListStyleSheets(FILE *out, int32_t aIndent = 0) override;
+  virtual void VerifyStyleTree() override;
 #endif
 
 #ifdef PR_LOGGING
   static PRLogModuleInfo* gLog;
 #endif
 
-  virtual void DisableNonTestMouseEvents(bool aDisable) MOZ_OVERRIDE;
+  virtual void DisableNonTestMouseEvents(bool aDisable) override;
 
-  virtual void UpdateCanvasBackground() MOZ_OVERRIDE;
+  virtual void UpdateCanvasBackground() override;
 
   virtual void AddCanvasBackgroundColorItem(nsDisplayListBuilder& aBuilder,
                                             nsDisplayList& aList,
                                             nsIFrame* aFrame,
                                             const nsRect& aBounds,
                                             nscolor aBackstopColor,
-                                            uint32_t aFlags) MOZ_OVERRIDE;
+                                            uint32_t aFlags) override;
 
   virtual void AddPrintPreviewBackgroundItem(nsDisplayListBuilder& aBuilder,
                                              nsDisplayList& aList,
                                              nsIFrame* aFrame,
-                                             const nsRect& aBounds) MOZ_OVERRIDE;
+                                             const nsRect& aBounds) override;
 
-  virtual nscolor ComputeBackstopColor(nsView* aDisplayRoot) MOZ_OVERRIDE;
+  virtual nscolor ComputeBackstopColor(nsView* aDisplayRoot) override;
 
-  virtual nsresult SetIsActive(bool aIsActive) MOZ_OVERRIDE;
+  virtual nsresult SetIsActive(bool aIsActive) override;
 
-  virtual bool GetIsViewportOverridden() MOZ_OVERRIDE { return mViewportOverridden; }
+  virtual bool GetIsViewportOverridden() override { return mViewportOverridden; }
 
-  virtual bool IsLayoutFlushObserver() MOZ_OVERRIDE
+  virtual bool IsLayoutFlushObserver() override
   {
     return GetPresContext()->RefreshDriver()->
       IsLayoutFlushObserver(this);
   }
 
-  virtual void LoadComplete() MOZ_OVERRIDE;
+  virtual void LoadComplete() override;
 
   void AddSizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf,
                               nsArenaMemoryStats *aArenaObjectsSize,
                               size_t *aPresShellSize,
                               size_t *aStyleSetsSize,
                               size_t *aTextRunsSize,
-                              size_t *aPresContextSize) MOZ_OVERRIDE;
+                              size_t *aPresContextSize) override;
   size_t SizeOfTextRuns(mozilla::MallocSizeOf aMallocSizeOf) const;
 
-  virtual void AddInvalidateHiddenPresShellObserver(nsRefreshDriver *aDriver) MOZ_OVERRIDE;
+  virtual void AddInvalidateHiddenPresShellObserver(nsRefreshDriver *aDriver) override;
 
   // This data is stored as a content property (nsGkAtoms::scrolling) on
   // mContentToScrollTo when we have a pending ScrollIntoView.
@@ -371,22 +371,22 @@ public:
     uint32_t   mContentToScrollToFlags;
   };
 
-  virtual void ScheduleImageVisibilityUpdate() MOZ_OVERRIDE;
+  virtual void ScheduleImageVisibilityUpdate() override;
 
-  virtual void RebuildImageVisibilityDisplayList(const nsDisplayList& aList) MOZ_OVERRIDE;
-  virtual void RebuildImageVisibility(nsRect* aRect = nullptr) MOZ_OVERRIDE;
+  virtual void RebuildImageVisibilityDisplayList(const nsDisplayList& aList) override;
+  virtual void RebuildImageVisibility(nsRect* aRect = nullptr) override;
 
-  virtual void EnsureImageInVisibleList(nsIImageLoadingContent* aImage) MOZ_OVERRIDE;
+  virtual void EnsureImageInVisibleList(nsIImageLoadingContent* aImage) override;
 
-  virtual void RemoveImageFromVisibleList(nsIImageLoadingContent* aImage) MOZ_OVERRIDE;
+  virtual void RemoveImageFromVisibleList(nsIImageLoadingContent* aImage) override;
 
-  virtual bool AssumeAllImagesVisible() MOZ_OVERRIDE;
+  virtual bool AssumeAllImagesVisible() override;
 
-  virtual void RecordShadowStyleChange(mozilla::dom::ShadowRoot* aShadowRoot) MOZ_OVERRIDE;
+  virtual void RecordShadowStyleChange(mozilla::dom::ShadowRoot* aShadowRoot) override;
 
   virtual void DispatchAfterKeyboardEvent(nsINode* aTarget,
                                           const mozilla::WidgetKeyboardEvent& aEvent,
-                                          bool aEmbeddedCancelled) MOZ_OVERRIDE;
+                                          bool aEmbeddedCancelled) override;
 
   void SetNextPaintCompressed() { mNextPaintCompressed = true; }
 
@@ -405,10 +405,13 @@ protected:
   nsresult DidCauseReflow();
   friend class nsAutoCauseReflowNotifier;
 
-  void DispatchTouchEvent(mozilla::WidgetEvent* aEvent,
-                          nsEventStatus* aStatus,
-                          nsPresShellEventCB* aEventCB,
-                          bool aTouchIsNew);
+  nsresult DispatchEventToDOM(mozilla::WidgetEvent* aEvent,
+                              nsEventStatus* aStatus,
+                              nsPresShellEventCB* aEventCB);
+  void DispatchTouchEventToDOM(mozilla::WidgetEvent* aEvent,
+                               nsEventStatus* aStatus,
+                               nsPresShellEventCB* aEventCB,
+                               bool aTouchIsNew);
 
   void     WillDoReflow();
 
@@ -460,12 +463,10 @@ protected:
 
   struct RenderingState {
     explicit RenderingState(PresShell* aPresShell)
-      : mXResolution(aPresShell->mXResolution)
-      , mYResolution(aPresShell->mYResolution)
+      : mResolution(aPresShell->mResolution)
       , mRenderFlags(aPresShell->mRenderFlags)
     { }
-    float mXResolution;
-    float mYResolution;
+    float mResolution;
     RenderFlags mRenderFlags;
   };
 
@@ -478,8 +479,7 @@ protected:
     ~AutoSaveRestoreRenderingState()
     {
       mPresShell->mRenderFlags = mOldState.mRenderFlags;
-      mPresShell->mXResolution = mOldState.mXResolution;
-      mPresShell->mYResolution = mOldState.mYResolution;
+      mPresShell->mResolution = mOldState.mResolution;
     }
 
     PresShell* mPresShell;
@@ -590,7 +590,7 @@ protected:
   class DelayedInputEvent : public DelayedEvent
   {
   public:
-    virtual void Dispatch() MOZ_OVERRIDE;
+    virtual void Dispatch() override;
 
   protected:
     DelayedInputEvent();
@@ -614,7 +614,7 @@ protected:
   // Check if aEvent is a mouse event and record the mouse location for later
   // synth mouse moves.
   void RecordMouseLocation(mozilla::WidgetGUIEvent* aEvent);
-  class nsSynthMouseMoveEvent MOZ_FINAL : public nsARefreshObserver {
+  class nsSynthMouseMoveEvent final : public nsARefreshObserver {
   public:
     nsSynthMouseMoveEvent(PresShell* aPresShell, bool aFromScroll)
       : mPresShell(aPresShell), mFromScroll(aFromScroll) {
@@ -628,7 +628,7 @@ protected:
     }
 
   public:
-    NS_INLINE_DECL_REFCOUNTING(nsSynthMouseMoveEvent, MOZ_OVERRIDE)
+    NS_INLINE_DECL_REFCOUNTING(nsSynthMouseMoveEvent, override)
 
     void Revoke() {
       if (mPresShell) {
@@ -637,7 +637,7 @@ protected:
         mPresShell = nullptr;
       }
     }
-    virtual void WillRefresh(mozilla::TimeStamp aTime) MOZ_OVERRIDE {
+    virtual void WillRefresh(mozilla::TimeStamp aTime) override {
       if (mPresShell) {
         nsRefPtr<PresShell> shell = mPresShell;
         shell->ProcessSynthMouseMoveEvent(mFromScroll);
@@ -690,7 +690,8 @@ protected:
   bool AdjustContextMenuKeyEvent(mozilla::WidgetMouseEvent* aEvent);
 
   //
-  bool PrepareToUseCaretPosition(nsIWidget* aEventWidget, nsIntPoint& aTargetPt);
+  bool PrepareToUseCaretPosition(nsIWidget* aEventWidget,
+                                 mozilla::LayoutDeviceIntPoint& aTargetPt);
 
   // Get the selected item and coordinates in device pixels relative to root
   // document's root view for element, first ensuring the element is onscreen
@@ -702,7 +703,7 @@ protected:
   void FireResizeEvent();
   static void AsyncResizeEventCallback(nsITimer* aTimer, void* aPresShell);
 
-  virtual void SynthesizeMouseMove(bool aFromScroll) MOZ_OVERRIDE;
+  virtual void SynthesizeMouseMove(bool aFromScroll) override;
 
   PresShell* GetRootPresShell();
 
@@ -718,13 +719,13 @@ protected:
   bool ScheduleReflowOffTimer();
 
   // Widget notificiations
-  virtual void WindowSizeMoveDone() MOZ_OVERRIDE;
-  virtual void SysColorChanged() MOZ_OVERRIDE { mPresContext->SysColorChanged(); }
-  virtual void ThemeChanged() MOZ_OVERRIDE { mPresContext->ThemeChanged(); }
-  virtual void BackingScaleFactorChanged() MOZ_OVERRIDE { mPresContext->UIResolutionChanged(); }
+  virtual void WindowSizeMoveDone() override;
+  virtual void SysColorChanged() override { mPresContext->SysColorChanged(); }
+  virtual void ThemeChanged() override { mPresContext->ThemeChanged(); }
+  virtual void BackingScaleFactorChanged() override { mPresContext->UIResolutionChanged(); }
 
-  virtual void PausePainting() MOZ_OVERRIDE;
-  virtual void ResumePainting() MOZ_OVERRIDE;
+  virtual void PausePainting() override;
+  virtual void ResumePainting() override;
 
   void UpdateImageVisibility();
   void UpdateActivePointerState(mozilla::WidgetGUIEvent* aEvent);
@@ -736,8 +737,6 @@ protected:
   static void MarkImagesInListVisible(const nsDisplayList& aList);
   void MarkImagesInSubtreeVisible(nsIFrame* aFrame, const nsRect& aRect);
 
-  void EvictTouches();
-
   // Methods for dispatching KeyboardEvent and BeforeAfterKeyboardEvent.
   void HandleKeyboardEvent(nsINode* aTarget,
                            mozilla::WidgetKeyboardEvent& aEvent,
@@ -759,7 +758,7 @@ protected:
   // A list of images that are visible or almost visible.
   nsTHashtable< nsRefPtrHashKey<nsIImageLoadingContent> > mVisibleImages;
 
-  nsresult SetResolutionImpl(float aXResolution, float aYResolution, bool aScaleToResolution);
+  nsresult SetResolutionImpl(float aResolution, bool aScaleToResolution);
 
 #ifdef DEBUG
   // The reflow root under which we're currently reflowing.  Null when
@@ -808,6 +807,9 @@ protected:
   nsCallbackEventRequest*   mFirstCallbackEventRequest;
   nsCallbackEventRequest*   mLastCallbackEventRequest;
 
+  // TouchManager
+  TouchManager              mTouchManager;
+
   // TouchCaret
   nsRefPtr<mozilla::TouchCaret> mTouchCaret;
   nsRefPtr<mozilla::SelectionCarets> mSelectionCarets;
@@ -833,6 +835,11 @@ protected:
 
   nscoord                   mLastAnchorScrollPositionY;
 
+  // Information about live content (which still stay in DOM tree).
+  // Used in case we need re-dispatch event after sending pointer event,
+  // when target of pointer event was deleted during executing user handlers.
+  nsCOMPtr<nsIContent>      mPointerEventTarget;
+
   // This is used to protect ourselves from triggering reflow while in the
   // middle of frame construction and the like... it really shouldn't be
   // needed, one hopes, but it is for now.
@@ -868,6 +875,9 @@ protected:
   // applied to rendered layers.
   bool                      mScaleToResolution : 1;
 
+  // Whether the last chrome-only escape key event is consumed.
+  bool                      mIsLastChromeOnlyEscapeKeyConsumed : 1;
+
   static bool               sDisableNonTestMouseEvents;
 };
 
