@@ -118,8 +118,8 @@ public:
 private:
   mutable bool mURIResolved;
 
-  URLValue(const URLValue& aOther) MOZ_DELETE;
-  URLValue& operator=(const URLValue& aOther) MOZ_DELETE;
+  URLValue(const URLValue& aOther) = delete;
+  URLValue& operator=(const URLValue& aOther) = delete;
 };
 
 struct ImageValue : public URLValue {
@@ -138,8 +138,10 @@ public:
   nsRefPtrHashtable<nsPtrHashKey<nsISupports>, imgRequestProxy> mRequests; 
 
   // Override AddRef and Release to not only log ourselves correctly, but
-  // also so that we delete correctly without a virtual destructor
-  NS_INLINE_DECL_REFCOUNTING(ImageValue)
+  // also so that we delete correctly without a virtual destructor (assuming
+  // callers always call *our* Release method and not our base class's).
+  NS_METHOD_(MozExternalRefCountType) AddRef();
+  NS_METHOD_(MozExternalRefCountType) Release();
 };
 
 struct GridNamedArea {
@@ -150,7 +152,7 @@ struct GridNamedArea {
   uint32_t mRowEnd;
 };
 
-struct GridTemplateAreasValue MOZ_FINAL {
+struct GridTemplateAreasValue final {
   // Parsed value
   nsTArray<GridNamedArea> mNamedAreas;
 
@@ -195,12 +197,12 @@ private:
   {
   }
 
-  GridTemplateAreasValue(const GridTemplateAreasValue& aOther) MOZ_DELETE;
+  GridTemplateAreasValue(const GridTemplateAreasValue& aOther) = delete;
   GridTemplateAreasValue&
-  operator=(const GridTemplateAreasValue& aOther) MOZ_DELETE;
+  operator=(const GridTemplateAreasValue& aOther) = delete;
 };
 
-class FontFamilyListRefCnt MOZ_FINAL : public FontFamilyList {
+class FontFamilyListRefCnt final : public FontFamilyList {
 public:
     FontFamilyListRefCnt()
         : FontFamilyList()
@@ -395,7 +397,7 @@ public:
   explicit nsCSSValue(nsCSSUnit aUnit = eCSSUnit_Null)
     : mUnit(aUnit)
   {
-    NS_ABORT_IF_FALSE(aUnit <= eCSSUnit_DummyInherit, "not a valueless unit");
+    MOZ_ASSERT(aUnit <= eCSSUnit_DummyInherit, "not a valueless unit");
   }
 
   nsCSSValue(int32_t aValue, nsCSSUnit aUnit);
@@ -500,36 +502,36 @@ public:
 
   int32_t GetIntValue() const
   {
-    NS_ABORT_IF_FALSE(mUnit == eCSSUnit_Integer ||
-                      mUnit == eCSSUnit_Enumerated ||
-                      mUnit == eCSSUnit_EnumColor,
-                      "not an int value");
+    MOZ_ASSERT(mUnit == eCSSUnit_Integer ||
+               mUnit == eCSSUnit_Enumerated ||
+               mUnit == eCSSUnit_EnumColor,
+               "not an int value");
     return mValue.mInt;
   }
 
   nsCSSKeyword GetKeywordValue() const
   {
-    NS_ABORT_IF_FALSE(mUnit == eCSSUnit_Enumerated, "not a keyword value");
+    MOZ_ASSERT(mUnit == eCSSUnit_Enumerated, "not a keyword value");
     return static_cast<nsCSSKeyword>(mValue.mInt);
   }
 
   float GetPercentValue() const
   {
-    NS_ABORT_IF_FALSE(mUnit == eCSSUnit_Percent, "not a percent value");
+    MOZ_ASSERT(mUnit == eCSSUnit_Percent, "not a percent value");
     return mValue.mFloat;
   }
 
   float GetFloatValue() const
   {
-    NS_ABORT_IF_FALSE(eCSSUnit_Number <= mUnit, "not a float value");
+    MOZ_ASSERT(eCSSUnit_Number <= mUnit, "not a float value");
     MOZ_ASSERT(!mozilla::IsNaN(mValue.mFloat));
     return mValue.mFloat;
   }
 
   float GetAngleValue() const
   {
-    NS_ABORT_IF_FALSE(eCSSUnit_Degree <= mUnit &&
-                 mUnit <= eCSSUnit_Turn, "not an angle value");
+    MOZ_ASSERT(eCSSUnit_Degree <= mUnit && mUnit <= eCSSUnit_Turn,
+               "not an angle value");
     return mValue.mFloat;
   }
 
@@ -538,7 +540,7 @@ public:
 
   nsAString& GetStringValue(nsAString& aBuffer) const
   {
-    NS_ABORT_IF_FALSE(UnitHasStringValue(), "not a string value");
+    MOZ_ASSERT(UnitHasStringValue(), "not a string value");
     aBuffer.Truncate();
     uint32_t len = NS_strlen(GetBufferValue(mValue.mString));
     mValue.mString->ToString(len, aBuffer);
@@ -547,7 +549,7 @@ public:
 
   const char16_t* GetStringBufferValue() const
   {
-    NS_ABORT_IF_FALSE(UnitHasStringValue(), "not a string value");
+    MOZ_ASSERT(UnitHasStringValue(), "not a string value");
     return GetBufferValue(mValue.mString);
   }
 
@@ -556,40 +558,40 @@ public:
 
   Array* GetArrayValue() const
   {
-    NS_ABORT_IF_FALSE(UnitHasArrayValue(), "not an array value");
+    MOZ_ASSERT(UnitHasArrayValue(), "not an array value");
     return mValue.mArray;
   }
 
   nsIURI* GetURLValue() const
   {
-    NS_ABORT_IF_FALSE(mUnit == eCSSUnit_URL || mUnit == eCSSUnit_Image,
-                 "not a URL value");
+    MOZ_ASSERT(mUnit == eCSSUnit_URL || mUnit == eCSSUnit_Image,
+               "not a URL value");
     return mUnit == eCSSUnit_URL ?
       mValue.mURL->GetURI() : mValue.mImage->GetURI();
   }
 
   nsCSSValueGradient* GetGradientValue() const
   {
-    NS_ABORT_IF_FALSE(mUnit == eCSSUnit_Gradient, "not a gradient value");
+    MOZ_ASSERT(mUnit == eCSSUnit_Gradient, "not a gradient value");
     return mValue.mGradient;
   }
 
   nsCSSValueTokenStream* GetTokenStreamValue() const
   {
-    NS_ABORT_IF_FALSE(mUnit == eCSSUnit_TokenStream, "not a token stream value");
+    MOZ_ASSERT(mUnit == eCSSUnit_TokenStream, "not a token stream value");
     return mValue.mTokenStream;
   }
 
   nsCSSValueSharedList* GetSharedListValue() const
   {
-    NS_ABORT_IF_FALSE(mUnit == eCSSUnit_SharedList, "not a shared list value");
+    MOZ_ASSERT(mUnit == eCSSUnit_SharedList, "not a shared list value");
     return mValue.mSharedList;
   }
 
   mozilla::FontFamilyList* GetFontFamilyListValue() const
   {
-    NS_ABORT_IF_FALSE(mUnit == eCSSUnit_FontFamilyList,
-                      "not a font family list value");
+    MOZ_ASSERT(mUnit == eCSSUnit_FontFamilyList,
+               "not a font family list value");
     NS_ASSERTION(mValue.mFontFamilyList != nullptr,
                  "font family list value should never be null");
     return mValue.mFontFamilyList;
@@ -616,27 +618,27 @@ public:
   {
     // Not allowing this for Image values, because if the caller takes
     // a ref to them they won't be able to delete them properly.
-    NS_ABORT_IF_FALSE(mUnit == eCSSUnit_URL, "not a URL value");
+    MOZ_ASSERT(mUnit == eCSSUnit_URL, "not a URL value");
     return mValue.mURL;
   }
 
   mozilla::css::ImageValue* GetImageStructValue() const
   {
-    NS_ABORT_IF_FALSE(mUnit == eCSSUnit_Image, "not an Image value");
+    MOZ_ASSERT(mUnit == eCSSUnit_Image, "not an Image value");
     return mValue.mImage;
   }
 
   mozilla::css::GridTemplateAreasValue* GetGridTemplateAreas() const
   {
-    NS_ABORT_IF_FALSE(mUnit == eCSSUnit_GridTemplateAreas,
-                      "not a grid-template-areas value");
+    MOZ_ASSERT(mUnit == eCSSUnit_GridTemplateAreas,
+               "not a grid-template-areas value");
     return mValue.mGridTemplateAreas;
   }
 
   const char16_t* GetOriginalURLValue() const
   {
-    NS_ABORT_IF_FALSE(mUnit == eCSSUnit_URL || mUnit == eCSSUnit_Image,
-                      "not a URL value");
+    MOZ_ASSERT(mUnit == eCSSUnit_URL || mUnit == eCSSUnit_Image,
+               "not a URL value");
     return GetBufferValue(mUnit == eCSSUnit_URL ?
                             mValue.mURL->mString :
                             mValue.mImage->mString);
@@ -769,7 +771,7 @@ protected:
   } mValue;
 };
 
-struct nsCSSValue::Array MOZ_FINAL {
+struct nsCSSValue::Array final {
 
   // return |Array| with reference count of zero
   static Array* Create(size_t aItemCount) {
@@ -777,12 +779,12 @@ struct nsCSSValue::Array MOZ_FINAL {
   }
 
   nsCSSValue& operator[](size_t aIndex) {
-    NS_ABORT_IF_FALSE(aIndex < mCount, "out of range");
+    MOZ_ASSERT(aIndex < mCount, "out of range");
     return mArray[aIndex];
   }
 
   const nsCSSValue& operator[](size_t aIndex) const {
-    NS_ABORT_IF_FALSE(aIndex < mCount, "out of range");
+    MOZ_ASSERT(aIndex < mCount, "out of range");
     return mArray[aIndex];
   }
 
@@ -837,7 +839,7 @@ private:
   nsCSSValue mArray[1];
 
   void* operator new(size_t aSelfSize, size_t aItemCount) CPP_THROW_NEW {
-    NS_ABORT_IF_FALSE(aItemCount > 0, "cannot have a 0 item count");
+    MOZ_ASSERT(aItemCount > 0, "cannot have a 0 item count");
     return ::operator new(aSelfSize + sizeof(nsCSSValue) * (aItemCount - 1));
   }
 
@@ -874,8 +876,8 @@ private:
 #undef CSSVALUE_LIST_FOR_EXTRA_VALUES
 
 private:
-  Array(const Array& aOther) MOZ_DELETE;
-  Array& operator=(const Array& aOther) MOZ_DELETE;
+  Array(const Array& aOther) = delete;
+  Array& operator=(const Array& aOther) = delete;
 };
 
 // Prefer nsCSSValue::Array for lists of fixed size.
@@ -906,14 +908,14 @@ private:
   // We don't want operator== or operator!= because they wouldn't be
   // null-safe, which is generally what we need.  Use |Equal| method
   // above instead.
-  bool operator==(nsCSSValueList const& aOther) const MOZ_DELETE;
-  bool operator!=(const nsCSSValueList& aOther) const MOZ_DELETE;
+  bool operator==(nsCSSValueList const& aOther) const = delete;
+  bool operator!=(const nsCSSValueList& aOther) const = delete;
 };
 
 // nsCSSValueList_heap differs from nsCSSValueList only in being
 // refcounted.  It should not be necessary to use this class directly;
 // it's an implementation detail of nsCSSValue.
-struct nsCSSValueList_heap MOZ_FINAL : public nsCSSValueList {
+struct nsCSSValueList_heap final : public nsCSSValueList {
   NS_INLINE_DECL_REFCOUNTING(nsCSSValueList_heap)
 
   size_t SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) const;
@@ -929,7 +931,7 @@ private:
 // a wrapper for the reference count and a pointer to the head of the
 // list, whereas the other list types (such as nsCSSValueList) do
 // not have such a wrapper.
-struct nsCSSValueSharedList MOZ_FINAL {
+struct nsCSSValueSharedList final {
   nsCSSValueSharedList()
     : mHead(nullptr)
   {
@@ -970,7 +972,7 @@ nsCSSValue::GetListValue()
   if (mUnit == eCSSUnit_List)
     return mValue.mList;
   else {
-    NS_ABORT_IF_FALSE(mUnit == eCSSUnit_ListDep, "not a pairlist value");
+    MOZ_ASSERT(mUnit == eCSSUnit_ListDep, "not a pairlist value");
     return mValue.mListDependent;
   }
 }
@@ -981,7 +983,7 @@ nsCSSValue::GetListValue() const
   if (mUnit == eCSSUnit_List)
     return mValue.mList;
   else {
-    NS_ABORT_IF_FALSE(mUnit == eCSSUnit_ListDep, "not a pairlist value");
+    MOZ_ASSERT(mUnit == eCSSUnit_ListDep, "not a pairlist value");
     return mValue.mListDependent;
   }
 }
@@ -1044,7 +1046,7 @@ struct nsCSSRect {
 // nsCSSRect_heap differs from nsCSSRect only in being
 // refcounted.  It should not be necessary to use this class directly;
 // it's an implementation detail of nsCSSValue.
-struct nsCSSRect_heap MOZ_FINAL : public nsCSSRect {
+struct nsCSSRect_heap final : public nsCSSRect {
   NS_INLINE_DECL_REFCOUNTING(nsCSSRect_heap)
 
   size_t SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) const;
@@ -1061,14 +1063,14 @@ private:
 inline nsCSSRect&
 nsCSSValue::GetRectValue()
 {
-  NS_ABORT_IF_FALSE(mUnit == eCSSUnit_Rect, "not a rect value");
+  MOZ_ASSERT(mUnit == eCSSUnit_Rect, "not a rect value");
   return *mValue.mRect;
 }
 
 inline const nsCSSRect&
 nsCSSValue::GetRectValue() const
 {
-  NS_ABORT_IF_FALSE(mUnit == eCSSUnit_Rect, "not a rect value");
+  MOZ_ASSERT(mUnit == eCSSUnit_Rect, "not a rect value");
   return *mValue.mRect;
 }
 
@@ -1139,7 +1141,7 @@ struct nsCSSValuePair {
 // nsCSSValuePair_heap differs from nsCSSValuePair only in being
 // refcounted.  It should not be necessary to use this class directly;
 // it's an implementation detail of nsCSSValue.
-struct nsCSSValuePair_heap MOZ_FINAL : public nsCSSValuePair {
+struct nsCSSValuePair_heap final : public nsCSSValuePair {
   // forward constructor
   nsCSSValuePair_heap(const nsCSSValue& aXValue, const nsCSSValue& aYValue)
       : nsCSSValuePair(aXValue, aYValue)
@@ -1230,7 +1232,7 @@ struct nsCSSValueTriplet {
 // nsCSSValueTriplet_heap differs from nsCSSValueTriplet only in being
 // refcounted.  It should not be necessary to use this class directly;
 // it's an implementation detail of nsCSSValue.
-struct nsCSSValueTriplet_heap MOZ_FINAL : public nsCSSValueTriplet {
+struct nsCSSValueTriplet_heap final : public nsCSSValueTriplet {
   // forward constructor
   nsCSSValueTriplet_heap(const nsCSSValue& aXValue, const nsCSSValue& aYValue, const nsCSSValue& aZValue)
     : nsCSSValueTriplet(aXValue, aYValue, aZValue)
@@ -1252,28 +1254,28 @@ private:
 inline nsCSSValuePair&
 nsCSSValue::GetPairValue()
 {
-  NS_ABORT_IF_FALSE(mUnit == eCSSUnit_Pair, "not a pair value");
+  MOZ_ASSERT(mUnit == eCSSUnit_Pair, "not a pair value");
   return *mValue.mPair;
 }
 
 inline const nsCSSValuePair&
 nsCSSValue::GetPairValue() const
 {
-  NS_ABORT_IF_FALSE(mUnit == eCSSUnit_Pair, "not a pair value");
+  MOZ_ASSERT(mUnit == eCSSUnit_Pair, "not a pair value");
   return *mValue.mPair;
 }
 
 inline nsCSSValueTriplet&
 nsCSSValue::GetTripletValue()
 {
-    NS_ABORT_IF_FALSE(mUnit == eCSSUnit_Triplet, "not a triplet value");
+    MOZ_ASSERT(mUnit == eCSSUnit_Triplet, "not a triplet value");
     return *mValue.mTriplet;
 }
 
 inline const nsCSSValueTriplet&
 nsCSSValue::GetTripletValue() const
 {
-    NS_ABORT_IF_FALSE(mUnit == eCSSUnit_Triplet, "not a triplet value");
+    MOZ_ASSERT(mUnit == eCSSUnit_Triplet, "not a triplet value");
     return *mValue.mTriplet;
 }
 
@@ -1305,14 +1307,14 @@ private:
   // We don't want operator== or operator!= because they wouldn't be
   // null-safe, which is generally what we need.  Use |Equal| method
   // above instead.
-  bool operator==(const nsCSSValuePairList& aOther) const MOZ_DELETE;
-  bool operator!=(const nsCSSValuePairList& aOther) const MOZ_DELETE;
+  bool operator==(const nsCSSValuePairList& aOther) const = delete;
+  bool operator!=(const nsCSSValuePairList& aOther) const = delete;
 };
 
 // nsCSSValuePairList_heap differs from nsCSSValuePairList only in being
 // refcounted.  It should not be necessary to use this class directly;
 // it's an implementation detail of nsCSSValue.
-struct nsCSSValuePairList_heap MOZ_FINAL : public nsCSSValuePairList {
+struct nsCSSValuePairList_heap final : public nsCSSValuePairList {
   NS_INLINE_DECL_REFCOUNTING(nsCSSValuePairList_heap)
 
   size_t SizeOfIncludingThis(mozilla::MallocSizeOf aMallocSizeOf) const;
@@ -1332,7 +1334,7 @@ nsCSSValue::GetPairListValue()
   if (mUnit == eCSSUnit_PairList)
     return mValue.mPairList;
   else {
-    NS_ABORT_IF_FALSE (mUnit == eCSSUnit_PairListDep, "not a pairlist value");
+    MOZ_ASSERT (mUnit == eCSSUnit_PairListDep, "not a pairlist value");
     return mValue.mPairListDependent;
   }
 }
@@ -1343,7 +1345,7 @@ nsCSSValue::GetPairListValue() const
   if (mUnit == eCSSUnit_PairList)
     return mValue.mPairList;
   else {
-    NS_ABORT_IF_FALSE (mUnit == eCSSUnit_PairListDep, "not a pairlist value");
+    MOZ_ASSERT (mUnit == eCSSUnit_PairListDep, "not a pairlist value");
     return mValue.mPairListDependent;
   }
 }
@@ -1377,7 +1379,7 @@ public:
   size_t SizeOfExcludingThis(mozilla::MallocSizeOf aMallocSizeOf) const;
 };
 
-struct nsCSSValueGradient MOZ_FINAL {
+struct nsCSSValueGradient final {
   nsCSSValueGradient(bool aIsRadial, bool aIsRepeating);
 
   // true if gradient is radial, false if it is linear
@@ -1474,11 +1476,11 @@ private:
   {
   }
 
-  nsCSSValueGradient(const nsCSSValueGradient& aOther) MOZ_DELETE;
-  nsCSSValueGradient& operator=(const nsCSSValueGradient& aOther) MOZ_DELETE;
+  nsCSSValueGradient(const nsCSSValueGradient& aOther) = delete;
+  nsCSSValueGradient& operator=(const nsCSSValueGradient& aOther) = delete;
 };
 
-struct nsCSSValueTokenStream MOZ_FINAL {
+struct nsCSSValueTokenStream final {
   nsCSSValueTokenStream();
 
 private:
@@ -1553,11 +1555,11 @@ public:
   nsTHashtable<nsRefPtrHashKey<mozilla::css::ImageValue> > mImageValues;
 
 private:
-  nsCSSValueTokenStream(const nsCSSValueTokenStream& aOther) MOZ_DELETE;
-  nsCSSValueTokenStream& operator=(const nsCSSValueTokenStream& aOther) MOZ_DELETE;
+  nsCSSValueTokenStream(const nsCSSValueTokenStream& aOther) = delete;
+  nsCSSValueTokenStream& operator=(const nsCSSValueTokenStream& aOther) = delete;
 };
 
-class nsCSSValueFloatColor MOZ_FINAL {
+class nsCSSValueFloatColor final {
 public:
   nsCSSValueFloatColor(float aComponent1, float aComponent2, float aComponent3,
                        float aAlpha)
@@ -1595,9 +1597,9 @@ private:
   float mComponent3;  // 0..1
   float mAlpha;       // 0..1
 
-  nsCSSValueFloatColor(const nsCSSValueFloatColor& aOther) MOZ_DELETE;
+  nsCSSValueFloatColor(const nsCSSValueFloatColor& aOther) = delete;
   nsCSSValueFloatColor& operator=(const nsCSSValueFloatColor& aOther)
-                                                                   MOZ_DELETE;
+                                                                   = delete;
 };
 
 struct nsCSSCornerSizes {
