@@ -11,7 +11,6 @@
 #include "nsMemory.h"
 #include "nsThreadUtils.h"
 #include "nsIClassInfoImpl.h"
-#include "nsIProgrammingLanguage.h"
 #include "Variant.h"
 
 #include "mozIStorageError.h"
@@ -88,13 +87,6 @@ public:
     return NS_OK;
   }
 
-  NS_IMETHODIMP
-  GetImplementationLanguage(uint32_t *_language) override
-  {
-    *_language = nsIProgrammingLanguage::CPLUSPLUS;
-    return NS_OK;
-  }
-
   NS_IMETHODIMP
   GetFlags(uint32_t *_flags) override
   {
@@ -840,6 +832,18 @@ Statement::GetBlob(uint32_t aIndex,
   return NS_OK;
 }
 
+NS_IMETHODIMP
+Statement::GetBlobAsString(uint32_t aIndex, nsAString& aValue)
+{
+  return DoGetBlobAsString(this, aIndex, aValue);
+}
+
+NS_IMETHODIMP
+Statement::GetBlobAsUTF8String(uint32_t aIndex, nsACString& aValue)
+{
+  return DoGetBlobAsString(this, aIndex, aValue);
+}
+
 NS_IMETHODIMP
 Statement::GetSharedUTF8String(uint32_t aIndex,
                                uint32_t *_length,
