@@ -21,9 +21,7 @@ using namespace mozilla::widget;
 
 static nsIMM32Handler* gIMM32Handler = nullptr;
 
-#ifdef PR_LOGGING
 PRLogModuleInfo* gIMM32Log = nullptr;
-#endif
 
 static UINT sWM_MSIME_MOUSE = 0; // mouse message for MSIME 98/2000
 
@@ -57,10 +55,8 @@ nsIMM32Handler::EnsureHandlerInstance()
 /* static */ void
 nsIMM32Handler::Initialize()
 {
-#ifdef PR_LOGGING
   if (!gIMM32Log)
     gIMM32Log = PR_NewLogModule("nsIMM32HandlerWidgets");
-#endif
 
   if (!sWM_MSIME_MOUSE) {
     sWM_MSIME_MOUSE = ::RegisterWindowMessage(RWM_MOUSE);
@@ -533,7 +529,6 @@ nsIMM32Handler::OnIMENotify(nsWindow* aWindow,
                             LPARAM lParam,
                             MSGResult& aResult)
 {
-#ifdef PR_LOGGING
   switch (wParam) {
     case IMN_CHANGECANDIDATE:
       PR_LOG(gIMM32Log, PR_LOG_ALWAYS,
@@ -606,7 +601,6 @@ nsIMM32Handler::OnIMENotify(nsWindow* aWindow,
          aWindow->GetWindowHandle()));
       break;
   }
-#endif // PR_LOGGING
 
   // not implement yet
   aResult.mConsumed = false;
@@ -1544,7 +1538,6 @@ PlatformToNSAttr(uint8_t aAttr)
   }
 }
 
-#ifdef PR_LOGGING
 static const char*
 GetRangeTypeName(uint32_t aRangeType)
 {
@@ -1563,7 +1556,6 @@ GetRangeTypeName(uint32_t aRangeType)
       return "UNKNOWN SELECTION TYPE!!";
   }
 }
-#endif
 
 void
 nsIMM32Handler::DispatchCompositionChangeEvent(nsWindow* aWindow,
@@ -1689,7 +1681,7 @@ nsIMM32Handler::GetCompositionString(const nsIMEContext &aIMEContext,
   long lRtn = ::ImmGetCompositionStringW(aIMEContext.get(), aIndex, nullptr, 0);
   if (lRtn < 0 ||
       !aCompositionString.SetLength((lRtn / sizeof(WCHAR)) + 1,
-                                    mozilla::fallible_t())) {
+                                    mozilla::fallible)) {
     PR_LOG(gIMM32Log, PR_LOG_ALWAYS,
       ("IMM32: GetCompositionString, FAILED by OOM\n"));
     return; // Error or out of memory.
@@ -1756,7 +1748,7 @@ nsIMM32Handler::ConvertToANSIString(const nsAFlatString& aStr, UINT aCodePage,
                                   nullptr, 0, nullptr, nullptr);
   NS_ENSURE_TRUE(len >= 0, false);
 
-  if (!aANSIStr.SetLength(len, mozilla::fallible_t())) {
+  if (!aANSIStr.SetLength(len, mozilla::fallible)) {
     PR_LOG(gIMM32Log, PR_LOG_ALWAYS,
       ("IMM32: ConvertToANSIString, FAILED by OOM\n"));
     return false;
@@ -1805,7 +1797,7 @@ nsIMM32Handler::GetCharacterRectOfSelectedTextAt(nsWindow* aWindow,
     aWindow->InitEvent(charRect, &point);
     aWindow->DispatchWindowEvent(&charRect);
     if (charRect.mSucceeded) {
-      aCharRect = charRect.mReply.mRect;
+      aCharRect = LayoutDevicePixel::ToUntyped(charRect.mReply.mRect);
       PR_LOG(gIMM32Log, PR_LOG_ALWAYS,
         ("IMM32: GetCharacterRectOfSelectedTextAt, aOffset=%lu, SUCCEEDED\n",
          aOffset));
@@ -1844,7 +1836,7 @@ nsIMM32Handler::GetCaretRect(nsWindow* aWindow, nsIntRect &aCaretRect)
       ("IMM32: GetCaretRect,  FAILED (NS_QUERY_CARET_RECT)\n"));
     return false;
   }
-  aCaretRect = caretRect.mReply.mRect;
+  aCaretRect = LayoutDevicePixel::ToUntyped(caretRect.mReply.mRect);
   PR_LOG(gIMM32Log, PR_LOG_ALWAYS,
     ("IMM32: GetCaretRect, SUCCEEDED, aCaretRect={ x: %ld, y: %ld, width: %ld, height: %ld }\n",
      aCaretRect.x, aCaretRect.y, aCaretRect.width, aCaretRect.height));
@@ -1956,7 +1948,7 @@ nsIMM32Handler::SetIMERelatedWindowsPosOnPlugin(nsWindow* aWindow,
   // Clip the plugin rect by the client rect of the window because composition
   // window needs to be specified the position in the client area.
   nsWindow* toplevelWindow = aWindow->GetTopLevelWindow(false);
-  nsIntRect pluginRectInScreen =
+  LayoutDeviceIntRect pluginRectInScreen =
     editorRectEvent.mReply.mRect + toplevelWindow->WidgetToScreenOffset();
   nsIntRect winRectInScreen;
   aWindow->GetClientBounds(winRectInScreen);
@@ -1974,7 +1966,7 @@ nsIMM32Handler::SetIMERelatedWindowsPosOnPlugin(nsWindow* aWindow,
   int32_t yMost = std::min(pluginRectInScreen.YMost(), winRectInScreen.YMost());
   clippedPluginRect.width = std::max(0, xMost - clippedPluginRect.x);
   clippedPluginRect.height = std::max(0, yMost - clippedPluginRect.y);
-  clippedPluginRect -= aWindow->WidgetToScreenOffset();
+  clippedPluginRect -= aWindow->WidgetToScreenOffsetUntyped();
 
   // Cover the plugin with native caret.  This prevents IME's window and plugin
   // overlap.
@@ -2013,10 +2005,10 @@ nsIMM32Handler::ResolveIMECaretPos(nsIWidget* aReferenceWidget,
     return;
 
   if (aReferenceWidget)
-    aOutRect.MoveBy(aReferenceWidget->WidgetToScreenOffset());
+    aOutRect.MoveBy(aReferenceWidget->WidgetToScreenOffsetUntyped());
 
   if (aNewOriginWidget)
-    aOutRect.MoveBy(-aNewOriginWidget->WidgetToScreenOffset());
+    aOutRect.MoveBy(-aNewOriginWidget->WidgetToScreenOffsetUntyped());
 }
 
 /* static */ nsresult
